<?xml version="1.0" encoding="UTF-8"?>

<!--
	* Licensed Materials - Property of IBM
	* 5724-C11, 5724-C13, 5724-D17
	* (C) Copyright IBM Corporation 1997, 2003. All Rights Reserved.
	* US Government Users Restricted Rights- Use, duplication or disclosure
	* restricted by GSA ADP Schedule Contract with IBM Corp.
-->

<xsl:stylesheet 
	version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:UML="href://org.omg/UML/1.3"
	xmlns:java="http://xml.apache.org/xslt/java"	
	exclude-result-prefixes="xalan java" >

	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>	

	<!-- 
	***************************************************************************************************************************************************************************
	Global variable definitions
	***************************************************************************************************************************************************************************
	-->

	<!-- *** Format utility variables *** -->

	<xsl:variable name="newline">
		<xsl:text>
		</xsl:text>
	</xsl:variable>	

	<!-- 
	***************************************************************************************************************************************************************************
	Converts the collaboration template 4.2 to the UML 1.3 metamodel by extending XMI 1.1 specification.
	***************************************************************************************************************************************************************************
	-->

	<!-- *** XSLT starts at the root template *** -->
	<xsl:template match="/" xmlns:UML='href://org.omg/UML/1.3'>

		<!-- *** write the DOCTYPE so that the validation can be done by the XML parsers *** -->
<xsl:text disable-output-escaping="yes">&lt;!DOCTYPE XMI SYSTEM 'UMLX13-11.dtd' > 
</xsl:text> 

		<!-- *** write the ROOT XMI Element *** -->
		<XMI xmi.version='1.1' xmlns:UML='href://org.omg/UML/1.3' timestamp='Wed Oct 30 05:53:46 2002'>
			<XMI.header>
				<XMI.documentation>
					<XMI.exporter>IBM.WBI.PD</XMI.exporter>
					<XMI.exporterVersion>4.2.0</XMI.exporterVersion>
				</XMI.documentation>
				<XMI.metamodel xmi.name = 'UML' xmi.version = '1.3'/>
			</XMI.header>

			<xsl:element name="XMI.content">
				<!-- *** Now write the Model content for the root element *** -->
				<xsl:apply-templates select="CollabTemplate" />	
			</xsl:element>
		</XMI>
	</xsl:template>

	<!-- *** the root template applies the CollabTemplate element *** -->
	<xsl:template match="CollabTemplate" >
		<xsl:element name="UML:Model">

			<xsl:attribute name="xmi.id">
				<xsl:value-of select="'G.0'" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="@Name" />
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isRoot">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isLeaf">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isAbstract">
				<xsl:value-of select="'false'" />
			</xsl:attribute>

			<xsl:apply-templates select="Scenarios" />				
		</xsl:element>

		<!--- *** now write the descriptions of all valid scenario elements *** -->
		<xsl:call-template name="writeDescriptions" />

		<xsl:call-template name="writeGUIDiagrams" />
	</xsl:template>

	<xsl:template match="Scenarios" >	
		<xsl:choose>			
			<xsl:when test="count(Scenario) &gt; 0 and count(Scenario[*]/GUIDiagrams/GUIDiagram[*]/NodesGUI/NodeGUI) &gt; 0">
				<xsl:element name="UML:Namespace.ownedElement">			
					<xsl:element name="UML:ActivityGraph">					
						<xsl:attribute name="xmi.id">
							<xsl:value-of select="'S.302.1114.56.1'" />
						</xsl:attribute>
						<xsl:attribute name="name">
							<xsl:value-of select="'Scenarios'" />						
						</xsl:attribute>
						<xsl:attribute name="visibility">
							<xsl:value-of select="'public'" />
						</xsl:attribute>
						<xsl:attribute name="isSpecification">
							<xsl:value-of select="'false'" />
						</xsl:attribute>
						<xsl:attribute name="context">
							<xsl:value-of select="'G.0'" />
						</xsl:attribute>


						<xsl:element name="UML:StateMachine.top">							
							<UML:CompositeState xmi.id = 'XX.30.1114.56.2' name = '{top}' visibility = 'public' isSpecification = 'false' isConcurrent = 'false' >
								<UML:CompositeState.subvertex>
									<xsl:for-each select="Scenario">
										<xsl:call-template name="writeScenarioNodes">
											<xsl:with-param name="ScenarioUniqueID" select="position() + 1" />
										</xsl:call-template>										
									</xsl:for-each>			
								</UML:CompositeState.subvertex>				
							</UML:CompositeState>
						</xsl:element>

						<xsl:element name="UML:StateMachine.transitions">
							<xsl:for-each select="Scenario">
								<xsl:call-template name="writeScenarioLinks" />
							</xsl:for-each>														
						</xsl:element>
					</xsl:element>

					<xsl:call-template name="writeStereotypes" />
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="UML:Namespace.ownedElement">			
					<xsl:element name="UML:StateMachine">					
						<xsl:attribute name="xmi.id">
							<xsl:value-of select="'S.302.1114.56.1'" />
						</xsl:attribute>
						<xsl:attribute name="name">
							<xsl:value-of select="'Scenarios'" />						
						</xsl:attribute>
						<xsl:attribute name="visibility">
							<xsl:value-of select="'public'" />
						</xsl:attribute>
						<xsl:attribute name="isSpecification">
							<xsl:value-of select="'false'" />
						</xsl:attribute>
						<xsl:attribute name="context">
							<xsl:value-of select="'G.0'" />
						</xsl:attribute>

						<xsl:element name="UML:StateMachine.top">							
						</xsl:element>

						<xsl:element name="UML:StateMachine.transitions">							
						</xsl:element>
					</xsl:element>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	
	<xsl:template name="writeDescriptions">			
		<xsl:if test="count(Scenarios/Scenario) &gt; 0 and count(Scenarios/Scenario[*]/GUIDiagrams/GUIDiagram[*]/NodesGUI/NodeGUI) &gt; 0">
			<xsl:for-each select="Scenarios/Scenario">				
				<!-- *** get the main diagram of the scenarion and start form there *** -->
				<xsl:variable name="scenarioIndex" select="position()" />
				<xsl:if test="count(GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]) = 1">
					<xsl:for-each select="GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]">						
						<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
						<xsl:call-template name="writeDiagramDescriptions">
							<xsl:with-param name="DiagramOwnerID" select="concat('G.', ($scenarioIndex + 1))" />
							<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
						</xsl:call-template>
					</xsl:for-each>	
				</xsl:if>								
			</xsl:for-each>						
		</xsl:if>
	</xsl:template>

	<xsl:template name="writeDiagramDescriptions">
	<xsl:param name="DiagramOwnerID" />
	<xsl:param name="nodeDiagram" />
		<xsl:for-each select="NodesGUI/*">
			<xsl:variable name="nNodeUniqID" select=".//NodeLinkGUI/@UniqIDRef" />
			<xsl:variable name="nodeDiagramNode" select="$nodeDiagram/DiagramBody/Nodes/*[.//ModelActivityElement/@UniqID = $nNodeUniqID]" />
			<xsl:variable name="strDesc">
				<xsl:choose>
					<xsl:when test='DecisionNode and not($nodeDiagramNode)'>
						<xsl:value-of select="normalize-space(DecisionNode/Desc)" />						
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="normalize-space($nodeDiagramNode/.//Node/ModelActivityElement/ModelElement/Desc)" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:if test="not($strDesc = '')" >
				<xsl:variable name="pd.id">
					<xsl:choose>
						<xsl:when test='DecisionNode and not($nodeDiagramNode)'>
							<xsl:value-of select="normalize-space(DecisionNode/@UniqID)" />						
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="normalize-space($nodeDiagramNode/.//Node/ModelActivityElement/@UniqID)" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<xsl:element name="UML:TaggedValue">
					<xsl:attribute name="xmi.id">
						<xsl:value-of select="concat('XX.T.D.', $DiagramOwnerID, $pd.id)" />
					</xsl:attribute>
					<xsl:attribute name="tag">
						<xsl:value-of select="'documentation'" />
					</xsl:attribute>
					<xsl:attribute name="value">
						<xsl:value-of select="$strDesc" />
					</xsl:attribute>
					<xsl:attribute name="modelElement">
						<xsl:value-of select="$pd.id" />
					</xsl:attribute>
				</xsl:element>
			</xsl:if> 
		</xsl:for-each>

		<xsl:for-each select="LinksGUI/*">
			<xsl:variable name="nLinkUniqID" select=".//NodeLinkGUI/@UniqIDRef" />
			<xsl:variable name="nodeDiagramLink" select="$nodeDiagram/DiagramBody/Links/*[.//ModelActivityElement/@UniqID = $nLinkUniqID]" />
			<xsl:variable name="strDesc">
				<xsl:choose>
					<xsl:when test='DecisionLink and not($nodeDiagramLink)'>
						<xsl:value-of select="normalize-space(DecisionLink/Desc)" />						
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="normalize-space($nodeDiagramLink/.//Link/ModelActivityElement/ModelElement/Desc)" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:if test="not($strDesc = '')" >
				<xsl:variable name="pd.id">
					<xsl:choose>
						<xsl:when test='DecisionLink and not($nodeDiagramLink)'>
							<xsl:value-of select="normalize-space(DecisionLink/@UniqID)" />						
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="normalize-space($nodeDiagramLink/.//Link/ModelActivityElement/@UniqID)" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:element name="UML:TaggedValue">
					<xsl:attribute name="xmi.id">
						<xsl:value-of select="concat('XX.T.D.', $DiagramOwnerID, $pd.id)" />
					</xsl:attribute>
					<xsl:attribute name="tag">
						<xsl:value-of select="'documentation'" />
					</xsl:attribute>
					<xsl:attribute name="value">
						<xsl:value-of select="$strDesc" />
					</xsl:attribute>
					<xsl:attribute name="modelElement">
						<xsl:value-of select="$pd.id" />
					</xsl:attribute>
				</xsl:element>
			</xsl:if> 
		</xsl:for-each>

		<!-- *** get the subdiagram info from the ModelSceanrio's corresponding diagram*** -->
		<xsl:for-each select="$nodeDiagram/DiagramBody/Nodes/SubDiagramNode | $nodeDiagram/DiagramBody/Nodes/IteratorNode">
			<!-- *** get the subdiagram info *** -->
			<xsl:variable name="subDiagramID" select=".//ModelActivityElement/@UniqID" />	
			<xsl:variable name="subDiagramName" select="concat('GUI__', CompositeNode/@DiagramRef)" />

			<xsl:if test="count(../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]) = 1">				
				<xsl:for-each select="../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]">
					<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
					<xsl:call-template name="writeDiagramDescriptions">
					<xsl:with-param name="DiagramOwnerID" select="$subDiagramID" />
						<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
					</xsl:call-template>
				</xsl:for-each>
			</xsl:if>				
		</xsl:for-each>

	</xsl:template>

	<xsl:template name="writeStereotypes">			
		<xsl:if test="count(Scenario) &gt; 0 and count(Scenario[*]/GUIDiagrams/GUIDiagram[*]/NodesGUI/NodeGUI) &gt; 0">
			
			<xsl:variable name="treeStereotypes">
				<xsl:for-each select="Scenario">				
					<!-- *** get the main diagram of the scenarion and start form there *** -->
					<xsl:variable name="scenarioIndex" select="position()" />
					<xsl:if test="count(GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]) = 1">					
						<xsl:for-each select="GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]">						
							<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
							<xsl:call-template name="writeDiagramStereotypes">
								<xsl:with-param name="DiagramOwnerID" select="concat('G.', ($scenarioIndex + 1))" />
								<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
							</xsl:call-template>
						</xsl:for-each>						
					</xsl:if>
				</xsl:for-each>
			</xsl:variable>
			<xsl:variable name="nodeStereotypes" select="xalan:nodeset($treeStereotypes)" />

			<xsl:if test="count($nodeStereotypes/Iterator) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'Iterator'" />
					<xsl:with-param name="strBaseClass" select="'Activity'" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/Iterator" />
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="count($nodeStereotypes/SubDiagram) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'SubDiagram'" />
					<xsl:with-param name="strBaseClass" select="'Activity'" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/SubDiagram" />
				</xsl:call-template>
			</xsl:if> 
<!-- *** not supported by other vendors 
			<xsl:if test="count($nodeStereotypes/SuccessEnd) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'SuccessEnd'" />
					<xsl:with-param name="strBaseClass" select="'FinalState'" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/SuccessEnd" />
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="count($nodeStereotypes/FailureEnd) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'FailureEnd'" />
					<xsl:with-param name="strBaseClass" select="'FinalState'" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/FailureEnd" />
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="count($nodeStereotypes/Break) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'Break'" />
					<xsl:with-param name="strBaseClass" select="'FinalState'" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/Break" />
				</xsl:call-template>
			</xsl:if>
-->
			<xsl:if test="count($nodeStereotypes/SyncOutboundService) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'SyncOutboundService'" />
					<xsl:with-param name="strBaseClass" select="'Activity'" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/SyncOutboundService" />
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="count($nodeStereotypes/ASyncOutboundService) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'ASyncOutboundService'" />
					<xsl:with-param name="strBaseClass" select="'Activity'" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/ASyncOutboundService" />
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="count($nodeStereotypes/InboundService) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'InboundService'" />
					<xsl:with-param name="strBaseClass" select="'Activity'" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/InboundService" />
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="count($nodeStereotypes/Transition) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'Transition'" />
					<xsl:with-param name="strBaseClass" select="''" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/Transition" />
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="count($nodeStereotypes/Message) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'Message'" />
					<xsl:with-param name="strBaseClass" select="''" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/Message" />
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="count($nodeStereotypes/Exception) &gt; 0">					
				<xsl:call-template name="writeStereotypeNode">
					<xsl:with-param name="strName" select="'Exception'" />
					<xsl:with-param name="strBaseClass" select="''" />
					<xsl:with-param name="nodeElements" select="$nodeStereotypes/Exception" />
				</xsl:call-template>
			</xsl:if>
		</xsl:if>
	</xsl:template>


	<xsl:template name="writeStereotypeNode">
	<xsl:param name="strName" />
	<xsl:param name="strBaseClass" />
	<xsl:param name="nodeElements" />
		<xsl:element name="UML:Stereotype">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="concat('S.T.1.', $strName)" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="$strName" />
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isRoot">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isLeaf">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isAbstract">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="icon">
				<xsl:value-of select="''" />
			</xsl:attribute>
			<xsl:attribute name="baseClass">
				<xsl:value-of select="$strBaseClass" />
			</xsl:attribute>
			<xsl:attribute name="extendedElement">
				<xsl:value-of select="' '" /><xsl:for-each select="$nodeElements"><xsl:value-of select="concat(normalize-space(@pd.id), ' ')" /></xsl:for-each>
			</xsl:attribute>
		</xsl:element>
	</xsl:template>



	<xsl:template name="writeDiagramStereotypes">
	<xsl:param name="DiagramOwnerID" />
	<xsl:param name="nodeDiagram" />
		<xsl:for-each select="NodesGUI/*">
			<xsl:variable name="nNodeUniqID" select=".//NodeLinkGUI/@UniqIDRef" />
			<xsl:variable name="nodeDiagramNode" select="$nodeDiagram/DiagramBody/Nodes/*[.//ModelActivityElement/@UniqID = $nNodeUniqID]" />

			<xsl:if test="$nodeDiagramNode">				
				<xsl:choose>
					<xsl:when test="name($nodeDiagramNode) = 'IteratorNode'">
						<xsl:element name="Iterator">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nNodeUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
					<xsl:when test="name($nodeDiagramNode) = 'SubDiagramNode'">
						<xsl:element name="SubDiagram">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nNodeUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
<!-- *** not supported by other vendors 
					<xsl:when test="name($nodeDiagramNode) = 'SuccessEndNode'">
						<xsl:element name="SuccessEnd">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nNodeUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
					<xsl:when test="name($nodeDiagramNode) = 'FailureEndNode'">
						<xsl:element name="FailureEnd">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nNodeUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
					<xsl:when test="name($nodeDiagramNode) = 'BreakNode'">
						<xsl:element name="Break">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nNodeUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
-->
					<xsl:when test="name($nodeDiagramNode) = 'OutboundServiceNode' and ($nodeDiagramNode/@IsSync = 'true')">
						<xsl:element name="SyncOutboundService">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nNodeUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
					<xsl:when test="name($nodeDiagramNode) = 'OutboundServiceNode' and not($nodeDiagramNode/@IsSync = 'true')">
						<xsl:element name="ASyncOutboundService">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nNodeUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
					<xsl:when test="name($nodeDiagramNode) = 'InboundServiceNode'">
						<xsl:element name="InboundService">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nNodeUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
				</xsl:choose>
			</xsl:if>
		</xsl:for-each>

		<xsl:for-each select="LinksGUI/*">
			<xsl:variable name="nLinkUniqID" select=".//NodeLinkGUI/@UniqIDRef" />
			<xsl:variable name="nodeDiagramLink" select="$nodeDiagram/DiagramBody/Links/*[.//ModelActivityElement/@UniqID = $nLinkUniqID]" />

			<xsl:if test="$nodeDiagramLink">				
				<xsl:choose>
					<xsl:when test="name($nodeDiagramLink) = 'TransitionLink'">
						<xsl:element name="Transition">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nLinkUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
					<xsl:when test="name($nodeDiagramLink) = 'MessageLink'">
						<xsl:element name="Message">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nLinkUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
					<xsl:when test="name($nodeDiagramLink) = 'ExceptionLink'">
						<xsl:element name="Exception">
							<xsl:attribute name="pd.id"> <xsl:value-of select="$nLinkUniqID" /> </xsl:attribute>
						</xsl:element>
					</xsl:when>
				</xsl:choose>
			</xsl:if>
		</xsl:for-each>

		<!-- *** get the subdiagram info from the ModelSceanrio's corresponding diagram*** -->		
		<xsl:for-each select="$nodeDiagram/DiagramBody/Nodes/SubDiagramNode | $nodeDiagram/DiagramBody/Nodes/IteratorNode">			
			<!-- *** get the subdiagram info *** -->
			<xsl:variable name="subDiagramID" select=".//ModelActivityElement/@UniqID" />	
			<xsl:variable name="subDiagramName" select="concat('GUI__', CompositeNode/@DiagramRef)" />		

			<xsl:if test="count(../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]) = 1">				
				<xsl:for-each select="../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]">
					<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
					<xsl:call-template name="writeDiagramStereotypes">
					<xsl:with-param name="DiagramOwnerID" select="$subDiagramID" />
						<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
					</xsl:call-template>
				</xsl:for-each>
			</xsl:if>				
		</xsl:for-each>

	</xsl:template>

	<xsl:template name="writeGUIDiagrams">		
		<xsl:if test="count(Scenarios/Scenario) &gt; 0 and count(Scenarios/Scenario[*]/GUIDiagrams/GUIDiagram[*]/NodesGUI/NodeGUI) &gt; 0">
			<xsl:for-each select="Scenarios/Scenario">
				<!-- *** get the main diagram of the scenarion and start form there *** -->
				<xsl:variable name="scenarioIndex" select="position()" />
				<xsl:if test="count(GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]) = 1">
					<xsl:for-each select="GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]">
						<xsl:call-template name="writeGUIDiagram">
							<xsl:with-param name="DiagramOwnerID" select="concat('G.', ($scenarioIndex + 1))" />							
						</xsl:call-template>	
					</xsl:for-each>	
				</xsl:if>								
			</xsl:for-each>						
		</xsl:if>
	</xsl:template>

	<xsl:template name="writeGUIDiagram">
	<xsl:param name="DiagramOwnerID" />
		<xsl:element name="UML:Diagram">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="concat('D.', $DiagramOwnerID)" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="substring-after(@Name, 'GUI__')" />						
			</xsl:attribute>
			<xsl:attribute name="toolName">
				<xsl:value-of select="'IBM WBI Process Designer 4.2.0'" />
			</xsl:attribute>
			<xsl:attribute name="diagramType">
				<xsl:value-of select="'ActivityDiagram'" />
			</xsl:attribute>
			<xsl:attribute name="style">
				<xsl:value-of select="''" />
			</xsl:attribute>
			<xsl:attribute name="owner">
				<xsl:value-of select="$DiagramOwnerID" />
			</xsl:attribute>

			<xsl:element name="UML:Diagram.element"> 
				<!-- *** position the diagram elements with preprocessing for nMinX and nMinY in the diagram*** -->
				<xsl:call-template name="repositionGUINodeElements">
					<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
					<xsl:with-param name="nNodesCount" select="count(NodesGUI/NodeGUI)" />
				</xsl:call-template>
			</xsl:element>
		</xsl:element>

		<!-- *** get the subdiagram info from the ModelSceanrio's corresponding diagram*** -->
		<xsl:variable name="currentDiagramName" select="substring-after(@Name, 'GUI__')" />
		<xsl:if test="count(../../ModelScenario/Diagram[@Name = $currentDiagramName]) = 1" >
			<xsl:for-each select="../../ModelScenario/Diagram[@Name = $currentDiagramName]">
				<xsl:for-each select="DiagramBody/Nodes/SubDiagramNode | DiagramBody/Nodes/IteratorNode">
					<!-- *** get the subdiagram info *** -->
					<xsl:variable name="subDiagramID" select=".//ModelActivityElement/@UniqID" />	
					<xsl:variable name="subDiagramName" select="concat('GUI__', CompositeNode/@DiagramRef)" />
					<xsl:if test="count(../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]) = 1">
						<xsl:for-each select="../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]">
							<xsl:call-template name="writeGUIDiagram" >
								<xsl:with-param name="DiagramOwnerID" select="$subDiagramID" />
							</xsl:call-template>
						</xsl:for-each>
					</xsl:if>				
				</xsl:for-each>
			</xsl:for-each>
		</xsl:if>

	</xsl:template>

	<xsl:template name="repositionGUINodeElements">
	<xsl:param name="DiagramOwnerID" />
	<xsl:param name="nNodesCount" />
	<xsl:param name="nNodeIndex" select="1" />
	<xsl:param name="nMinX" select="0" />
	<xsl:param name="nMinY" select="0" />
		<xsl:choose>
			<xsl:when test="$nNodeIndex = 1 and ($nNodeIndex &lt;= $nNodesCount)">
				<xsl:call-template name="repositionGUINodeElements">
					<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
					<xsl:with-param name="nNodesCount" select="$nNodesCount" />
					<xsl:with-param name="nNodeIndex" select="$nNodeIndex + 1" />
					<xsl:with-param name="nMinX" select="NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@X" />
					<xsl:with-param name="nMinY" select="NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@Y" />
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="$nNodeIndex &lt;= $nNodesCount">
				<xsl:choose>
					<xsl:when test="($nMinX &gt; NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@X) and ($nMinY &gt; NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@Y)">		
						<xsl:call-template name="repositionGUINodeElements">
							<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
							<xsl:with-param name="nNodesCount" select="$nNodesCount" />
							<xsl:with-param name="nNodeIndex" select="$nNodeIndex + 1" />
							<xsl:with-param name="nMinX" select="NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@X" />
							<xsl:with-param name="nMinY" select="NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@Y" />
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="($nMinX &gt; NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@X)">		
						<xsl:call-template name="repositionGUINodeElements">
							<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
							<xsl:with-param name="nNodesCount" select="$nNodesCount" />
							<xsl:with-param name="nNodeIndex" select="$nNodeIndex + 1" />
							<xsl:with-param name="nMinX" select="NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@X" />
							<xsl:with-param name="nMinY" select="$nMinY" />
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="($nMinY &gt; NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@Y)">		
						<xsl:call-template name="repositionGUINodeElements">
							<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
							<xsl:with-param name="nNodesCount" select="$nNodesCount" />
							<xsl:with-param name="nNodeIndex" select="$nNodeIndex + 1" />
							<xsl:with-param name="nMinX" select="$nMinX" />
							<xsl:with-param name="nMinY" select="NodesGUI/NodeGUI[$nNodeIndex]/GUICoord2/GUICoord[1]/@Y" />
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="repositionGUINodeElements">
							<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
							<xsl:with-param name="nNodesCount" select="$nNodesCount" />
							<xsl:with-param name="nNodeIndex" select="$nNodeIndex + 1" />
							<xsl:with-param name="nMinX" select="$nMinX" />
							<xsl:with-param name="nMinY" select="$nMinY" />
						</xsl:call-template>						
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="repositionGUILinkElements">
					<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />					
					<xsl:with-param name="nLinksCount" select="count(LinksGUI/LinkGUI)" />					
					<xsl:with-param name="nMinX" select="$nMinX" />
					<xsl:with-param name="nMinY" select="$nMinY" />
				</xsl:call-template>														
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="repositionGUILinkElements">
	<xsl:param name="DiagramOwnerID" />
	<xsl:param name="nLinksCount" />
	<xsl:param name="nLinkIndex" select="1" />
	<xsl:param name="nMinX" select="0" />
	<xsl:param name="nMinY" select="0" />		

		<xsl:variable name="treeMinCoordIndex">
			<xsl:choose>
				<xsl:when test= "($nLinkIndex &gt;= 1) and ($nLinkIndex &lt;= $nLinksCount)">
					<xsl:variable name="nMaxCoords" select="count(LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord)" />
					<xsl:choose>
						<xsl:when test="(LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[1]/@X &lt; LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nMaxCoords]/@X) and
										(LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[1]/@Y &lt; LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nMaxCoords]/@Y)" >
							<xsl:element name="pd.min.x.index">1</xsl:element>				
							<xsl:element name="pd.min.y.index">1</xsl:element>				
						</xsl:when>
						<xsl:when test="(LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[1]/@X &lt; LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nMaxCoords]/@X)" >
							<xsl:element name="pd.min.x.index">1</xsl:element>				
							<xsl:element name="pd.min.y.index">
								<xsl:value-of select="$nMaxCoords" />
							</xsl:element>				
						</xsl:when>
						<xsl:when test="(LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[1]/@Y &lt; LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nMaxCoords]/@Y)" >
							<xsl:element name="pd.min.x.index">
								<xsl:value-of select="$nMaxCoords" />
							</xsl:element>				
							<xsl:element name="pd.min.y.index">1</xsl:element>				
						</xsl:when>
						<xsl:otherwise>
							<xsl:element name="pd.min.x.index">
								<xsl:value-of select="$nMaxCoords" />
							</xsl:element>				
							<xsl:element name="pd.min.y.index">
								<xsl:value-of select="$nMaxCoords" />
							</xsl:element>				
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>					
					<xsl:element name="pd.min.x.index">1</xsl:element>				
					<xsl:element name="pd.min.y.index">1</xsl:element>				
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="nodeMinCoordIndex" select="xalan:nodeset($treeMinCoordIndex)" />	


		<xsl:choose>
			<xsl:when test="$nLinkIndex = 1 and ($nLinkIndex &lt;= $nLinksCount) and $nMinX = 0 and $nMinY = 0">
				<xsl:call-template name="repositionGUILinkElements">
					<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
					<xsl:with-param name="nLinksCount" select="$nLinksCount" />
					<xsl:with-param name="nLinkIndex" select="$nLinkIndex + 1" />
					<xsl:with-param name="nMinX" select="LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.x.index]/@X" />
					<xsl:with-param name="nMinY" select="LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.y.index]/@Y" />
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="$nLinkIndex &lt;= $nLinksCount">
				<xsl:choose>
					<xsl:when test="($nMinX &gt; LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.x.index]/@X) and ($nMinY &gt; LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.y.index]/@Y)">		
						<xsl:call-template name="repositionGUILinkElements">
							<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
							<xsl:with-param name="nLinksCount" select="$nLinksCount" />
							<xsl:with-param name="nLinkIndex" select="$nLinkIndex + 1" />
							<xsl:with-param name="nMinX" select="LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.x.index]/@X" />
							<xsl:with-param name="nMinY" select="LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.y.index]/@Y" />
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="($nMinX &gt; LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.x.index]/@X)">		
						<xsl:call-template name="repositionGUILinkElements">
							<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
							<xsl:with-param name="nLinksCount" select="$nLinksCount" />
							<xsl:with-param name="nLinkIndex" select="$nLinkIndex + 1" />
							<xsl:with-param name="nMinX" select="LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.x.index]/@X" />
							<xsl:with-param name="nMinY" select="$nMinY" />
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="($nMinY &gt; LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.y.index]/@Y)">		
						<xsl:call-template name="repositionGUILinkElements">
							<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
							<xsl:with-param name="nLinksCount" select="$nLinksCount" />
							<xsl:with-param name="nLinkIndex" select="$nLinkIndex + 1" />
							<xsl:with-param name="nMinX" select="$nMinX" />
							<xsl:with-param name="nMinY" select="LinksGUI/LinkGUI[$nLinkIndex]/GUICoords/GUICoord[$nodeMinCoordIndex/pd.min.y.index]/@Y" />
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="repositionGUILinkElements">
							<xsl:with-param name="DiagramOwnerID" select="$DiagramOwnerID" />
							<xsl:with-param name="nLinksCount" select="$nLinksCount" />
							<xsl:with-param name="nLinkIndex" select="$nLinkIndex + 1" />
							<xsl:with-param name="nMinX" select="$nMinX" />
							<xsl:with-param name="nMinY" select="$nMinY" />
						</xsl:call-template>
					</xsl:otherwise>					
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>


				<xsl:variable name="currentDiagramName" select="substring-after(@Name, 'GUI__')" />
				<xsl:variable name="nodeModelDiagram" select="../../ModelScenario/Diagram[@Name = $currentDiagramName]" />				

				<xsl:for-each select="NodesGUI/NodeGUI">


					<xsl:variable name="nNodeUniqID" select=".//NodeLinkGUI/@UniqIDRef" />					
					<xsl:variable name="treeSizeScaleFactor">
						<xsl:variable name="nodeModel" select="$nodeModelDiagram/DiagramBody/Nodes/*[.//ModelActivityElement/@UniqID = $nNodeUniqID]" />
						<xsl:choose>
							<xsl:when test="name($nodeModel) = 'InitialNode' or name($nodeModel) = 'SuccessEndNode' or name($nodeModel) = 'FailureEndNode' or name($nodeModel) = 'BreakNode'">
								<xsl:element name="pd.size.scale.x">3</xsl:element>
								<xsl:element name="pd.size.scale.y">2.75</xsl:element>
							</xsl:when>
							<xsl:when test="name($nodeModel) = 'OutboundServiceNode' or name($nodeModel) = 'InboundServiceNode'">
								<xsl:element name="pd.size.scale.x">10</xsl:element>
								<xsl:element name="pd.size.scale.y">3</xsl:element>
							</xsl:when>
							<xsl:when test="name($nodeModel) = 'IteratorNode' or name($nodeModel) = 'SubDiagramNode'">
								<xsl:element name="pd.size.scale.x">5</xsl:element>
								<xsl:element name="pd.size.scale.y">2</xsl:element>
							</xsl:when>
							<xsl:otherwise>
								<xsl:element name="pd.size.scale.x">5</xsl:element>
								<xsl:element name="pd.size.scale.y">2</xsl:element>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>

					<xsl:variable name="nodeSizeScaleFactor" select="xalan:nodeset($treeSizeScaleFactor)" />
					<xsl:variable name="centerX" select="round((GUICoord2/GUICoord[2]/@X - GUICoord2/GUICoord[1]/@X) div 2)" />
					<xsl:variable name="centerY" select="round((GUICoord2/GUICoord[2]/@Y - GUICoord2/GUICoord[1]/@Y) div 2)" />
					<xsl:variable name="deltaX" select="3 * (GUICoord2/GUICoord[1]/@X +  $centerX - $nMinX)" />
					<xsl:variable name="deltaY" select="3 * (GUICoord2/GUICoord[1]/@Y + $centerY - $nMinY)" />
					

					<xsl:element name="UML:DiagramElement">
						<xsl:attribute name="xmi.id">
							<xsl:value-of select="concat('XX.D.', $DiagramOwnerID, NodeLinkGUI/@UniqIDRef)" />
						</xsl:attribute>
						<xsl:attribute name="geometry">
							<xsl:value-of select="concat(' ', GUICoord2/GUICoord[1]/@X + $deltaX, ', ', GUICoord2/GUICoord[1]/@Y + $deltaY, ', ', (GUICoord2/GUICoord[2]/@X - GUICoord2/GUICoord[1]/@X) * $nodeSizeScaleFactor/pd.size.scale.x, ', ', round((GUICoord2/GUICoord[2]/@Y - GUICoord2/GUICoord[1]/@Y) * $nodeSizeScaleFactor/pd.size.scale.y), ',')" />						
						</xsl:attribute>
						<xsl:attribute name="style">
							<xsl:value-of select="'FillColor.Blue= 255,FillColor.Green= 255,FillColor.Red= 255,FillColor.Transparent=0,Font.Blue= 0,Font.Green= 0,Font.Red= 0,Font.FaceName=Arial,Font.Size= 10,Font.Bold=0,Font.Italic=0,Font.Strikethrough=0,Font.Underline=0,LineColor.Blue= 51,LineColor.Green= 0,LineColor.Red= 153,'" />
						</xsl:attribute>
						<xsl:attribute name="subject">
							<xsl:value-of select="NodeLinkGUI/@UniqIDRef" />
						</xsl:attribute>
					</xsl:element>						
				</xsl:for-each>

				<xsl:for-each select="LinksGUI/LinkGUI">					
					<xsl:element name="UML:DiagramElement">
						<xsl:attribute name="xmi.id">
							<xsl:value-of select="concat('XX.D.', $DiagramOwnerID, NodeLinkGUI/@UniqIDRef)" />
						</xsl:attribute>
						<xsl:attribute name="geometry">
							<xsl:variable name="nMaxCoords" select="count(GUICoords/GUICoord)" />
							<xsl:choose>
								<xsl:when test="GUICoords/GUICoord[$nMaxCoords]/@X &gt; GUICoords/GUICoord[1]/@X" >
									<xsl:choose>
										<xsl:when test="GUICoords/GUICoord[$nMaxCoords]/@Y &gt; GUICoords/GUICoord[1]/@Y" >
											<xsl:call-template name="writeSpecialLinkGeometry">
												<xsl:with-param name="nMinX" select="GUICoords/GUICoord[1]/@X" />
												<xsl:with-param name="nMaxX" select="GUICoords/GUICoord[$nMaxCoords]/@X" />
												<xsl:with-param name="nMinY" select="GUICoords/GUICoord[1]/@Y" />
												<xsl:with-param name="nMaxY" select="GUICoords/GUICoord[$nMaxCoords]/@Y" />
												<xsl:with-param name="nDiagramMinX" select="$nMinX" />
												<xsl:with-param name="nDiagramMinY" select="$nMinY" />
											</xsl:call-template>
										</xsl:when>
										<xsl:otherwise>
											<xsl:call-template name="writeSpecialLinkGeometry">
												<xsl:with-param name="nMinX" select="GUICoords/GUICoord[1]/@X" />
												<xsl:with-param name="nMaxX" select="GUICoords/GUICoord[$nMaxCoords]/@X" />
												<xsl:with-param name="nMinY" select="GUICoords/GUICoord[$nMaxCoords]/@Y" />
												<xsl:with-param name="nMaxY" select="GUICoords/GUICoord[1]/@Y" />
												<xsl:with-param name="nDiagramMinX" select="$nMinX" />
												<xsl:with-param name="nDiagramMinY" select="$nMinY" />
											</xsl:call-template>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
								<xsl:otherwise>
									<xsl:choose>
										<xsl:when test="GUICoords/GUICoord[$nMaxCoords]/@Y &gt; GUICoords/GUICoord[1]/@Y" >
											<xsl:call-template name="writeSpecialLinkGeometry">
												<xsl:with-param name="nMinX" select="GUICoords/GUICoord[$nMaxCoords]/@X" />
												<xsl:with-param name="nMaxX" select="GUICoords/GUICoord[1]/@X" />
												<xsl:with-param name="nMinY" select="GUICoords/GUICoord[1]/@Y" />
												<xsl:with-param name="nMaxY" select="GUICoords/GUICoord[$nMaxCoords]/@Y" />
												<xsl:with-param name="nDiagramMinX" select="$nMinX" />
												<xsl:with-param name="nDiagramMinY" select="$nMinY" />
											</xsl:call-template>
										</xsl:when>
										<xsl:otherwise>
											<xsl:call-template name="writeSpecialLinkGeometry">
												<xsl:with-param name="nMinX" select="GUICoords/GUICoord[$nMaxCoords]/@X" />
												<xsl:with-param name="nMaxX" select="GUICoords/GUICoord[1]/@X" />
												<xsl:with-param name="nMinY" select="GUICoords/GUICoord[$nMaxCoords]/@Y" />
												<xsl:with-param name="nMaxY" select="GUICoords/GUICoord[1]/@Y" />
												<xsl:with-param name="nDiagramMinX" select="$nMinX" />
												<xsl:with-param name="nDiagramMinY" select="$nMinY" />
											</xsl:call-template>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:attribute>
						<xsl:attribute name="style">
							<xsl:value-of select="'Transition'" />
						</xsl:attribute>
						<xsl:attribute name="subject">
							<xsl:value-of select="NodeLinkGUI/@UniqIDRef" />
						</xsl:attribute>
					</xsl:element>						

				</xsl:for-each>						
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="writeSpecialLinkGeometry">
	<xsl:param name="nMinX" />
	<xsl:param name="nMaxX" />
	<xsl:param name="nMinY" />
	<xsl:param name="nMaxY" />
	<xsl:param name="nDiagramMinX" />
	<xsl:param name="nDiagramMinY" />
		<xsl:choose>
			<xsl:when test="(not($nMaxX = $nMinX)) and not($nMaxY = $nMinY)">
				<xsl:variable name="centerX" select="round(($nMaxX - $nMinX) div 2)" />									
				<xsl:variable name="centerY" select="round(($nMaxY - $nMinY) div 2)" />									
				<xsl:variable name="deltaX" select="3 * ($nMinX +  $centerX - $nDiagramMinX)" />
				<xsl:variable name="deltaY" select="3 * ($nMinY +  $centerY - $nDiagramMinY)" />
				<xsl:value-of select="concat(' ', $nMinX + $deltaX, ', ', $nMinY + $deltaY, ', ', ($nMaxX - $nMinX) * 3, ', ', ($nMaxY - $nMinY) * 3, ',')" />						
			</xsl:when>
			<xsl:when test="not($nMaxX = $nMinX)">									
				<xsl:variable name="centerX" select="round(($nMaxX - $nMinX) div 2)" />									
				<xsl:variable name="deltaX" select="3 * ($nMinX +  $centerX - $nDiagramMinX)" />
				<xsl:variable name="deltaY" select="3 * ($nMinY - $nDiagramMinY)" />
				<xsl:value-of select="concat(' ', $nMinX + $deltaX, ', ', $nMinY + $deltaY, ', ', ($nMaxX - $nMinX) * 3, ', ', 10 * 3, ',')" />						
			</xsl:when>
			<xsl:when test="not($nMaxY = $nMinY)">
				<xsl:variable name="centerY" select="round(($nMaxY - $nMinY) div 2)" />
				<xsl:variable name="deltaX" select="3 * ($nMinX - $nDiagramMinX)" />
				<xsl:variable name="deltaY" select="3 * ($nMinY + $centerY - $nDiagramMinY)" />
			
			<!--
					<MinX>
					<xsl:value-of select="$nMinX" />
					</MinX>
					<MinY>
					<xsl:value-of select="$nMinY" />
					</MinY>
					<centerX>
					<xsl:value-of select="$centerY + $deltaY" />
					</centerX>
					<centerY>
					<xsl:value-of select="$centerY" />
					</centerY>
					<DMinX>
					<xsl:value-of select="$nDiagramMinX" />
					</DMinX>
					<DMinY>
					<xsl:value-of select="$nDiagramMinY" />
					</DMinY>
					-->

					

				<xsl:value-of select="concat(' ', $nMinX + $deltaX, ', ', $nMinY + $deltaY, ', ', 10 * 3, ', ', ($nMaxY - $nMinY) * 3, ',')" />						
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="deltaX" select="3 * ($nMinX - $nDiagramMinX)" />
				<xsl:variable name="deltaY" select="3 * ($nMinY - $nDiagramMinY)" />
				<xsl:value-of select="concat(' ', $nMinX + $deltaX, ', ', $nMinY + $deltaY, ', ', 10 * 3, ', ', 10 * 3, ',')" />						
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="writeScenarioNodes">
	<xsl:param name="ScenarioUniqueID" />
		<xsl:element name="UML:SubactivityState">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="concat('G.', $ScenarioUniqueID)" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="@Name" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isConcurrent">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isDynamic">
				<xsl:value-of select="'false'" />
			</xsl:attribute>

			<UML:SubactivityState.dynamicArguments>
				<UML:ArgListsExpression 
					language = '' body = '' />
			</UML:SubactivityState.dynamicArguments>
			<UML:SubactivityState.dynamicMultiplicity>
				<UML:Multiplicity >
					<UML:Multiplicity.range>
						<UML:MultiplicityRange 
							lower = '0' upper = '0' />
					</UML:Multiplicity.range>
				</UML:Multiplicity>
			</UML:SubactivityState.dynamicMultiplicity>
			<UML:CompositeState.subvertex>
				<!-- *** get the main diagram of the scenarion and start form there *** -->

				<xsl:if test="count(GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]) = 1">					
					<xsl:for-each select="GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]">					
						<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
						<xsl:call-template name="writeDiagramNodes">
							<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
						</xsl:call-template>
					</xsl:for-each>
				</xsl:if>				
			</UML:CompositeState.subvertex>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeScenarioLinks">
		<xsl:if test="count(GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]) = 1">
			<xsl:for-each select="GUIDiagrams/GUIDiagram[not(contains(@Name, '.'))]">
				<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
				<xsl:call-template name="writeDiagramLinks">
					<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
				</xsl:call-template>
			</xsl:for-each>
		</xsl:if>				
	</xsl:template>

	<xsl:template name="writeDiagramNodes">
	<xsl:param name="nodeDiagram" />	
		<xsl:for-each select="NodesGUI/*">			
			<xsl:variable name="nNodeUniqID" select=".//NodeLinkGUI/@UniqIDRef" />
			<xsl:variable name="nodeDiagramNode" select="$nodeDiagram/DiagramBody/Nodes/*[.//ModelActivityElement/@UniqID = $nNodeUniqID]" />
			<xsl:call-template name="writeNode">				
				<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="writeNode">
	<xsl:param name="nodeDiagramNode" />
		<xsl:choose>
			<xsl:when test="not($nodeDiagramNode) and DecisionNode">
				<xsl:call-template name="writeDecisionNode" />
			</xsl:when>
			<xsl:when test="name($nodeDiagramNode)='InitialNode'">
				<xsl:call-template name="writeInitialNode">
					<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />					
				</xsl:call-template>								
			</xsl:when>
			<xsl:when test="name($nodeDiagramNode)='ActionNode'">
				<xsl:call-template name="writeActionNode">
					<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />					
				</xsl:call-template>								
			</xsl:when>
			<xsl:when test="name($nodeDiagramNode)='SuccessEndNode'">
				<xsl:call-template name="writeEndNode">
					<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />					
					<xsl:with-param name="strLabel" select="'SuccessEnd'" />
				</xsl:call-template>								
			</xsl:when>
			<xsl:when test="name($nodeDiagramNode)='FailureEndNode'">
				<xsl:call-template name="writeEndNode">
					<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />					
					<xsl:with-param name="strLabel" select="'FailureEnd'" />
				</xsl:call-template>								
			</xsl:when>
			<xsl:when test="name($nodeDiagramNode)='OutboundServiceNode'">
				<xsl:call-template name="writeOutboundServiceNode">
					<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />					
				</xsl:call-template>								
			</xsl:when>
			<xsl:when test="name($nodeDiagramNode)='InboundServiceNode'">
				<xsl:call-template name="writeInboundServiceNode">
					<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />					
				</xsl:call-template>								
			</xsl:when>
			<xsl:when test="name($nodeDiagramNode)='IteratorNode'">
				<xsl:call-template name="writeIteratorNode">
					<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />					
				</xsl:call-template>								
			</xsl:when>
			<xsl:when test="name($nodeDiagramNode)='SubDiagramNode'">
				<xsl:call-template name="writeSubDiagramNode">
					<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />					
				</xsl:call-template>								
			</xsl:when>
			<xsl:when test="name($nodeDiagramNode)='BreakNode'">
				<xsl:call-template name="writeBreakNode">
					<xsl:with-param name="nodeDiagramNode" select="$nodeDiagramNode" />					
				</xsl:call-template>								
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="writeDiagramLinks">
	<xsl:param name="nodeDiagram" />			
		<xsl:for-each select="LinksGUI/*">
			<xsl:variable name="nLinkUniqID" select=".//NodeLinkGUI/@UniqIDRef" />
			<xsl:variable name="nodeDiagramLink" select="$nodeDiagram/DiagramBody/Links/*[.//ModelActivityElement/@UniqID = $nLinkUniqID]" />
			<xsl:call-template name="writeLink">				
				<xsl:with-param name="nodeDiagramLink" select="$nodeDiagramLink" />
			</xsl:call-template>
		</xsl:for-each>

		<xsl:for-each select="$nodeDiagram/DiagramBody/Nodes/SubDiagramNode | $nodeDiagram/DiagramBody/Nodes/IteratorNode">
			<!-- *** get the subdiagram info *** -->
			<xsl:variable name="subDiagramName" select="concat('GUI__', CompositeNode/@DiagramRef)" />

			<xsl:if test="count(../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]) = 1">
				<xsl:for-each select="../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]">
					<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
					<xsl:call-template name="writeDiagramLinks">
						<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
					</xsl:call-template>
				</xsl:for-each>
			</xsl:if>				
		</xsl:for-each>


	</xsl:template>

	<xsl:template name="writeLink">
	<xsl:param name="nodeDiagramLink" />			
		<xsl:choose>
			<xsl:when test="not($nodeDiagramLink) and DecisionLink">
				<xsl:call-template name="writeDecisionSourceLink">
					<xsl:with-param name="nodeDiagramLink" select="$nodeDiagramLink" />				
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="name($nodeDiagramLink)='TransitionLink'">
				<xsl:call-template name="writeTransitionLink" >				
					<xsl:with-param name="nodeDiagramLink" select="$nodeDiagramLink" />
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="name($nodeDiagramLink)='MessageLink'">
				<xsl:call-template name="writeTransitionLink" >
					<xsl:with-param name="nodeDiagramLink" select="$nodeDiagramLink" />				
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="name($nodeDiagramLink)='ExceptionLink'">
				<xsl:call-template name="writeTransitionLink" >				
					<xsl:with-param name="nodeDiagramLink" select="$nodeDiagramLink" />
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="writeDecisionSourceLink">
		<xsl:element name="UML:Transition">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="DecisionLink/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after(DecisionLink/@UniqID, '_')" />				
				<xsl:value-of select="concat(translate(DecisionLink/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="source">
				<xsl:value-of select="Connect/@From" />						
			</xsl:attribute>			
			<xsl:attribute name="target">
				<xsl:value-of select="Connect/@To" />						
			</xsl:attribute>			
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeTransitionLink">
	<xsl:param name="nodeDiagramLink" />	
		<xsl:variable name="strCondition">		
			<xsl:choose>
				<xsl:when test="name($nodeDiagramLink) = 'TransitionLink'">
					<xsl:value-of select="$nodeDiagramLink/GuardCondition" />
				</xsl:when>
				<xsl:when test="name($nodeDiagramLink) = 'ExceptionLink'">
					<xsl:value-of select="$nodeDiagramLink/ExceptionName" />
				</xsl:when>
				<xsl:otherwise>
				<xsl:value-of select="''" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:element name="UML:Transition">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$nodeDiagramLink/.//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($nodeDiagramLink/.//ModelActivityElement/@UniqID, '_')" />				
				<xsl:value-of select="concat(translate($nodeDiagramLink/.//ModelActivityElement/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="source">
				<xsl:choose>
					<xsl:when test="Connect">
						<xsl:value-of select="Connect/@From" />						
					</xsl:when>
					<xsl:otherwise>					
						<xsl:value-of select="$nodeDiagramLink/.//Connect/@From" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>			
			<xsl:attribute name="target">
				<xsl:choose>
					<xsl:when test="Connect">
						<xsl:value-of select="Connect/@To" />						
					</xsl:when>
					<xsl:otherwise>					
						<xsl:value-of select="$nodeDiagramLink/.//Connect/@To" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<UML:Transition.guard>
				<UML:Guard xmi.id = 'XX.7.1320.19.8' 
				  name = '' visibility = 'public' isSpecification = 'false' >
				<xsl:attribute name="xmi.id">
					<xsl:value-of select="concat('T.G.', $nodeDiagramLink/.//ModelActivityElement/@UniqID)" />
				</xsl:attribute>
				  <UML:Guard.expression>
					<UML:BooleanExpression 
					  language = '' >
						<xsl:attribute name="body">
							<xsl:value-of select="$strCondition" />
						</xsl:attribute>
					</UML:BooleanExpression >					
				  </UML:Guard.expression>
				</UML:Guard>
			</UML:Transition.guard>						
		</xsl:element>
	</xsl:template>


	<xsl:template name="writeInitialNode">
	<xsl:param name="nodeDiagramNode" />
		<xsl:element name="UML:Pseudostate">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$nodeDiagramNode/.//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($nodeDiagramNode/.//ModelActivityElement/@UniqID, '_')" />				
				<xsl:value-of select="concat(translate($nodeDiagramNode/.//ModelActivityElement/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="kind">
				<xsl:value-of select="'initial'" />
			</xsl:attribute>			
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeActionNode">
	<xsl:param name="nodeDiagramNode" />
		<xsl:variable name="pd.id" select="$nodeDiagramNode/.//ModelActivityElement/@UniqID" />		
		<xsl:element name="UML:ActionState">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$pd.id" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($pd.id, '_')" />				
				<xsl:value-of select="concat(translate($nodeDiagramNode/.//ModelActivityElement/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isDynamic">
				<xsl:value-of select="'false'" />
			</xsl:attribute>

			<UML:ActionState.dynamicArguments>
				<UML:ArgListsExpression 
					language = '' body = '' />
			</UML:ActionState.dynamicArguments>
			<UML:ActionState.dynamicMultiplicity>
				<UML:Multiplicity >
					<UML:Multiplicity.range>
						<UML:MultiplicityRange 
							lower = '0' upper = '0' />
					</UML:Multiplicity.range>
				</UML:Multiplicity>
			</UML:ActionState.dynamicMultiplicity>

            <UML:State.doActivity>
                <UML:ActionSequence 
                  name = '' visibility = 'public' isSpecification = 'false' 
                  isAsynchronous = 'false' >
                  <UML:Action.recurrence>
                    <UML:IterationExpression 
                      language = '' body = '' />
                  </UML:Action.recurrence>
                  <UML:Action.target>
                    <UML:ObjectSetExpression 
                      language = '' body = '' />
                  </UML:Action.target>
                  <UML:Action.script>
                    <UML:ActionExpression 
                      language = '' body = '' />
                  </UML:Action.script>
                  <UML:ActionSequence.action>
                    <UML:UninterpretedAction 
                      name = '' visibility = 'public' isSpecification = 'false' 
                      isAsynchronous = 'false' >
					  <xsl:attribute name="xmi.id">  <xsl:value-of select="concat('XX.A.D.', $pd.id)" /> </xsl:attribute> 
                      <UML:Action.recurrence>
                        <UML:IterationExpression 
                          language = '' body = '1' />
                      </UML:Action.recurrence>
                      <UML:Action.target>
                        <UML:ObjectSetExpression 
                          language = '' body = '' />
                      </UML:Action.target>
                      <UML:Action.script>
                        <UML:ActionExpression>
							<xsl:attribute name="language">  <xsl:value-of select="$nodeDiagramNode/CodeFragment/@Type" /> </xsl:attribute> 							
							<UML:Expression.body>
								<xsl:value-of select="$nodeDiagramNode/CodeFragment" /> 								
							</UML:Expression.body>                          
						</UML:ActionExpression>
                      </UML:Action.script>
                    </UML:UninterpretedAction>
                  </UML:ActionSequence.action>
                </UML:ActionSequence>
              </UML:State.doActivity>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeDecisionNode">
		<xsl:variable name="pd.id" select="DecisionNode/@UniqID" />		
		<xsl:element name="UML:Pseudostate">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$pd.id" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($pd.id, '_')" />				
				<xsl:value-of select="concat(translate(DecisionNode/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="kind">
				<xsl:value-of select="'branch'" />
			</xsl:attribute>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeOutboundServiceNode">
	<xsl:param name="nodeDiagramNode" />
		<xsl:element name="UML:ActionState">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$nodeDiagramNode/.//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($nodeDiagramNode/.//ModelActivityElement/@UniqID, '_')" />				
				<xsl:value-of select="concat(translate($nodeDiagramNode/.//ModelActivityElement/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isDynamic">
				<xsl:value-of select="'false'" />
			</xsl:attribute>

			<UML:ActionState.dynamicArguments>
				<UML:ArgListsExpression 
					language = '' body = '' />
			</UML:ActionState.dynamicArguments>
			<UML:ActionState.dynamicMultiplicity>
				<UML:Multiplicity >
					<UML:Multiplicity.range>
						<UML:MultiplicityRange 
							lower = '0' upper = '0' />
					</UML:Multiplicity.range>
				</UML:Multiplicity>
			</UML:ActionState.dynamicMultiplicity>

		</xsl:element>
	</xsl:template>

	<xsl:template name="writeInboundServiceNode">
	<xsl:param name="nodeDiagramNode" />
		<xsl:element name="UML:ActionState">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$nodeDiagramNode/.//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($nodeDiagramNode/.//ModelActivityElement/@UniqID, '_')" />				
				<xsl:value-of select="concat(translate($nodeDiagramNode/.//ModelActivityElement/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isDynamic">
				<xsl:value-of select="'false'" />
			</xsl:attribute>

			<UML:ActionState.dynamicArguments>
				<UML:ArgListsExpression 
					language = '' body = '' />
			</UML:ActionState.dynamicArguments>
			<UML:ActionState.dynamicMultiplicity>
				<UML:Multiplicity >
					<UML:Multiplicity.range>
						<UML:MultiplicityRange 
							lower = '0' upper = '0' />
					</UML:Multiplicity.range>
				</UML:Multiplicity>
			</UML:ActionState.dynamicMultiplicity>

		</xsl:element>
	</xsl:template>

	<xsl:template name="writeIteratorNode">
	<xsl:param name="nodeDiagramNode" />
	<xsl:param name="nodeGUIDiagram" />
		<xsl:element name="UML:SubactivityState">		
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$nodeDiagramNode/.//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($nodeDiagramNode/.//ModelActivityElement/@UniqID, '_')" />				
				<xsl:value-of select="concat(translate($nodeDiagramNode/.//ModelActivityElement/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isConcurrent">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isDynamic">
				<xsl:value-of select="'false'" />
			</xsl:attribute>

			<UML:SubactivityState.dynamicArguments>
				<UML:ArgListsExpression 
					language = '' body = '' />
			</UML:SubactivityState.dynamicArguments>
			<UML:SubactivityState.dynamicMultiplicity>
				<UML:Multiplicity >
					<UML:Multiplicity.range>
						<UML:MultiplicityRange 
							lower = '0' upper = '0' />
					</UML:Multiplicity.range>
				</UML:Multiplicity>
			</UML:SubactivityState.dynamicMultiplicity>
			<UML:CompositeState.subvertex>
				<!-- *** get the main diagram of the scenarion and start form there *** -->
				<xsl:variable name="subDiagramName" select="concat('GUI__', $nodeDiagramNode/CompositeNode/@DiagramRef)" />

				<xsl:if test="count(../../../GUIDiagram[@Name = $subDiagramName]) = 1">
					<xsl:for-each select="../../../GUIDiagram[@Name = $subDiagramName]">
						<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
						<xsl:call-template name="writeDiagramNodes">
							<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
						</xsl:call-template>
					</xsl:for-each>
				</xsl:if>				
			</UML:CompositeState.subvertex>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeSubDiagramNode">
	<xsl:param name="nodeDiagramNode" />
		<xsl:element name="UML:SubactivityState">		
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$nodeDiagramNode/.//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($nodeDiagramNode/.//ModelActivityElement/@UniqID, '_')" />				
				<xsl:value-of select="concat(translate($nodeDiagramNode/.//ModelActivityElement/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isConcurrent">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
			<xsl:attribute name="isDynamic">
				<xsl:value-of select="'false'" />
			</xsl:attribute>

			<UML:SubactivityState.dynamicArguments>
				<UML:ArgListsExpression 
					language = '' body = '' />
			</UML:SubactivityState.dynamicArguments>
			<UML:SubactivityState.dynamicMultiplicity>
				<UML:Multiplicity >
					<UML:Multiplicity.range>
						<UML:MultiplicityRange 
							lower = '0' upper = '0' />
					</UML:Multiplicity.range>
				</UML:Multiplicity>
			</UML:SubactivityState.dynamicMultiplicity>
			<UML:CompositeState.subvertex>
				<!-- *** get the subdiagram info *** -->
				<xsl:variable name="subDiagramName" select="concat('GUI__', $nodeDiagramNode/CompositeNode/@DiagramRef)" />
				

				<xsl:if test="count(../../../GUIDiagram[@Name = $subDiagramName]) = 1">
					<xsl:for-each select="../../../GUIDiagram[@Name = $subDiagramName]">					
						<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />						
						<xsl:call-template name="writeDiagramNodes">
							<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
						</xsl:call-template>
					</xsl:for-each>
				</xsl:if>				
			</UML:CompositeState.subvertex>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeEndNode">
	<xsl:param name="nodeDiagramNode" />
	<xsl:param name="strLabel" />
		<xsl:element name="UML:FinalState">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$nodeDiagramNode/.//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($nodeDiagramNode/.//ModelActivityElement/@UniqID, '_')" />				
				<xsl:value-of select="concat(translate($nodeDiagramNode/.//ModelActivityElement/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="xmi.label">
				<xsl:value-of select="$strLabel" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeBreakNode">
	<xsl:param name="nodeDiagramNode" />
		<xsl:element name="UML:FinalState">
			<xsl:attribute name="xmi.id">
				<xsl:value-of select="$nodeDiagramNode/.//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:variable name='UniqID' select="substring-after($nodeDiagramNode/.//ModelActivityElement/@UniqID, '_')" />				
				<xsl:value-of select="concat(translate($nodeDiagramNode/.//ModelActivityElement/Label, '&#10;', ' '), ' ', $UniqID)" />						
			</xsl:attribute>
			<xsl:attribute name="xmi.label">
				<xsl:value-of select="'Break'" />						
			</xsl:attribute>
			<xsl:attribute name="visibility">
				<xsl:value-of select="'public'" />
			</xsl:attribute>
			<xsl:attribute name="isSpecification">
				<xsl:value-of select="'false'" />
			</xsl:attribute>
		</xsl:element>
	</xsl:template>

</xsl:stylesheet>

