<?xml version="1.0" encoding="UTF-8"?>

<!--
	* Licensed Materials - Property of IBM
	* 5724-C11, 5724-C13, 5724-D17
	* (C) Copyright IBM Corporation 1997, 2003. All Rights Reserved.
	* US Government Users Restricted Rights- Use, duplication or disclosure
	* restricted by GSA ADP Schedule Contract with IBM Corp.
-->

<xsl:stylesheet 
	version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:UML="href://org.omg/UML/1.3"
	xmlns:java="http://xml.apache.org/xslt/java"	
	exclude-result-prefixes="UML xalan java" >

	<xsl:output method="xml" version="1.0" encoding="UTF-8" standalone="no" indent="yes"/>	

	<!-- 
	***************************************************************************************************************************************************************************
	Global variables
	***************************************************************************************************************************************************************************
-->

	<!-- *** constants *** -->
	<xsl:variable name="strTemplateVersion" select="'4.2.0'" />
	<xsl:variable name="strTemplateFormatVersion" select="'4.2.0'" />
	<xsl:variable name="strScenarioVersion" select="'4.2.0'" />

	<!-- *** Format utility variables *** -->
	<xsl:variable name="newline">
	<xsl:text>
	</xsl:text>
	</xsl:variable>	

	<!-- Create Java Object instance -->
	<!-- UniqueID - obtain unique ID --> 
	<xsl:variable name="objXSLTUtils" select="java:com.ibm.btools.itools.cwconverter.xslt.ext.XSLTUtils.new()"/>	
    


	<!-- 
	***************************************************************************************************************************************************************************
	Converts the more generic UML 1.3 metamodel by extending XMI 1.1 specification to the collaboration template 4.2 
	***************************************************************************************************************************************************************************
-->

	<!-- *** XSLT starts at the root template *** -->
	<xsl:template match="/">

		<!-- *** write the DTD for our root element CollabTemplate *** -->
		<xsl:text disable-output-escaping="yes">
			&lt;!DOCTYPE CollabTemplate [
			&lt;!--
			 Base Elements and Attributes
			-->
			&lt;!ELEMENT Title (#PCDATA)>
			&lt;!ELEMENT Desc (#PCDATA)>
			&lt;!ELEMENT Value (#PCDATA)>
			&lt;!ELEMENT Default (#PCDATA)>
			&lt;!ELEMENT CreateTime (#PCDATA)>
			&lt;!ATTLIST CreateTime Format CDATA #REQUIRED>
			&lt;!ELEMENT ModifyTime (#PCDATA)>
			&lt;!ATTLIST ModifyTime Format CDATA #REQUIRED>
			&lt;!ELEMENT GUICoord EMPTY>
			&lt;!ATTLIST GUICoord X CDATA #REQUIRED>
			&lt;!ATTLIST GUICoord Y CDATA #REQUIRED>
			&lt;!ELEMENT GUICoord2 (GUICoord,GUICoord)>
			&lt;!ELEMENT GUICoords (GUICoord,GUICoord+)>
			&lt;!ELEMENT Color EMPTY>
			&lt;!ATTLIST Color R CDATA #REQUIRED>
			&lt;!ATTLIST Color G CDATA #REQUIRED>
			&lt;!ATTLIST Color B CDATA #REQUIRED>
			&lt;!ELEMENT GUIFont (FaceName,PointSize,Height,Color)>
			&lt;!ATTLIST GUIFont Italic CDATA #REQUIRED>
			&lt;!ATTLIST GUIFont UnderLine CDATA #REQUIRED>
			&lt;!ATTLIST GUIFont StrikeOut CDATA #REQUIRED>
			&lt;!ELEMENT FaceName (#PCDATA)>
			&lt;!ELEMENT PointSize (#PCDATA)>
			&lt;!ELEMENT Height (#PCDATA)>

			&lt;!--
			 Header section
			-->
			&lt;!ELEMENT Header (Title,Desc,CreateTime,ModifyTime)>

			&lt;!--
			 Collaboration Document
			-->
			&lt;!ELEMENT CollabTemplate (Header,TemplateDefinition,GUITemplateDefinition,Scenarios,Messages)>
			&lt;!ATTLIST CollabTemplate Name CDATA #REQUIRED>
			&lt;!ATTLIST CollabTemplate Version CDATA #REQUIRED>
			&lt;!ATTLIST CollabTemplate FormatVersion CDATA #REQUIRED>

			&lt;!--
			 Collaboration Definition section
			-->
			&lt;!ELEMENT TemplateDefinition (Desc,Properties,MinTransactionLevel,Package,GeneratedVars,BusObjRefs)>
			&lt;!ELEMENT MinTransactionLevel EMPTY>
			&lt;!ATTLIST MinTransactionLevel Level (None|Minimum|Best|Stringent) "None">
			&lt;!ELEMENT Package (#PCDATA)>
			&lt;!ELEMENT GeneratedVars (#PCDATA)>
			&lt;!ELEMENT BusObjRefs (BusObjRef)*>
			&lt;!ELEMENT BusObjRef (ConnectorName,Verbs)>
			&lt;!ATTLIST BusObjRef Name CDATA #REQUIRED>
			&lt;!ATTLIST BusObjRef BusObjType CDATA #REQUIRED>
			&lt;!ATTLIST BusObjRef IsRequired (Yes|No) #REQUIRED>
			&lt;!ATTLIST BusObjRef IsConfigured (Yes|No) #REQUIRED>
			&lt;!ELEMENT ConnectorName (#PCDATA)>
			&lt;!ELEMENT Verbs (Verb)*>
			&lt;!ELEMENT Verb (ScenarioName)>
			&lt;!ATTLIST Verb Name CDATA #REQUIRED>
			&lt;!ATTLIST Verb IsSubscribeEvent (Yes|No) #REQUIRED>
			&lt;!ELEMENT ScenarioName (#PCDATA)>

			&lt;!--
			 GUI Collaboration Definition section
			-->
			&lt;!ELEMENT GUITemplateDefinition (GUIProperties,GUIImports,GUIUserSection1,GUIUserSection2,GUIUserSection3,GUIVariables)>
			&lt;!ELEMENT GUIProperties (GeneralProperties?,BOProbeProperties?)>
			&lt;!ELEMENT GeneralProperties (GUIProperty)*>
			&lt;!ELEMENT BOProbeProperties (GUIProperty)*>
			&lt;!ELEMENT GUIProperty (GUIPropertyType,Desc,MaxStringLength,MaxMultipleValues,DefaultValues,PossibleValues)>
			&lt;!ATTLIST GUIProperty Name CDATA #REQUIRED>
			&lt;!ELEMENT GUIPropertyType (#PCDATA)>
			&lt;!ELEMENT MaxStringLength (#PCDATA)>
			&lt;!ELEMENT MaxMultipleValues (#PCDATA)>
			&lt;!ELEMENT DefaultValues (Values,Ranges)>
			&lt;!ELEMENT PossibleValues (Values,Ranges)>
			&lt;!ELEMENT Values (GUIPropertyValue)*>
			&lt;!ELEMENT Ranges (Range)*>
			&lt;!ELEMENT Range (Min,Max)*>
			&lt;!ELEMENT GUIPropertyValue (#PCDATA)>
			&lt;!ELEMENT Min (#PCDATA)>
			&lt;!ELEMENT Max (#PCDATA)>
			&lt;!ELEMENT GUIImports (GUIImport)*>
			&lt;!ELEMENT GUIImport (ImportName,Comment)>
			&lt;!ELEMENT ImportName (#PCDATA)>
			&lt;!ELEMENT Comment (#PCDATA)>
			&lt;!ELEMENT GUIUserSection1 (#PCDATA)>
			&lt;!ELEMENT GUIUserSection2 (#PCDATA)>
			&lt;!ELEMENT GUIUserSection3 (#PCDATA)>
			&lt;!ELEMENT GUIVariables (GUIVariable)*>
			&lt;!ELEMENT GUIVariable (VarType,ObjectType?,VarName,VarValue,FirstStepInit?,VarModifier1,VarModifier2,VarModifier3,Comment)>
			&lt;!ELEMENT VarType (#PCDATA)>
			&lt;!ELEMENT ObjectType (#PCDATA)>
			&lt;!ATTLIST ObjectType NeedsTwoStepInit (Yes|No) #REQUIRED>
			&lt;!ELEMENT FirstStepInit (#PCDATA)>
			&lt;!ELEMENT VarName (#PCDATA)>
			&lt;!ELEMENT VarValue (#PCDATA)>
			&lt;!ELEMENT VarModifier1 (#PCDATA)>
			&lt;!ELEMENT VarModifier2 (#PCDATA)>
			&lt;!ELEMENT VarModifier3 (#PCDATA)>

			&lt;!--
			 Scenario section
			-->
			&lt;!ELEMENT Scenarios (Scenario)*>
			&lt;!ELEMENT Scenario (Header,GUIDefinition,GUIDiagrams,ModelScenario)>
			&lt;!ATTLIST Scenario Name CDATA #REQUIRED>
			&lt;!ATTLIST Scenario Version CDATA #REQUIRED>
			&lt;!ATTLIST Scenario DiagramRef IDREF #REQUIRED>
			&lt;!ATTLIST Scenario DiagramRefGUI IDREF #REQUIRED>
			&lt;!ELEMENT GUIDefinition (Desc,GUIUserSection1,GUIUserSection2,GUIVariables)>

			&lt;!--
			 GUI Diagram section
			-->
			&lt;!ELEMENT GUIDiagrams (GUIDiagram)*>
			&lt;!ELEMENT GUIDiagram (Header,GUIProperties,GUISettings,NodesGUI,LinksGUI,Texts)>
			&lt;!ATTLIST GUIDiagram Name ID #REQUIRED>
			&lt;!ATTLIST GUIDiagram Version CDATA #REQUIRED>
			&lt;!ELEMENT ModelScenario (Diagram)*>

			&lt;!--
			 GUI Diagram Settings section
			-->
			&lt;!ELEMENT GUISettings (DiagramPermission,PageSettings,ComponentView,Grid)>
			&lt;!ELEMENT DiagramPermission EMPTY>
			&lt;!ATTLIST DiagramPermission Value (Yes|No) #REQUIRED>
			&lt;!ELEMENT PageSettings EMPTY>
			&lt;!ATTLIST PageSettings SizeX CDATA #REQUIRED>
			&lt;!ATTLIST PageSettings SizeY CDATA #REQUIRED>
			&lt;!ATTLIST PageSettings BoundVisible (Yes|No) #REQUIRED>
			&lt;!ELEMENT ComponentView EMPTY>
			&lt;!ATTLIST ComponentView Type (Yes|No) #REQUIRED>
			&lt;!ATTLIST ComponentView UIDs (Yes|No) #REQUIRED>
			&lt;!ATTLIST ComponentView Labels (Yes|No) #REQUIRED>
			&lt;!ELEMENT Grid (Color)>
			&lt;!ATTLIST Grid Visible (Yes|No) #REQUIRED>
			&lt;!ATTLIST Grid Snap (Yes|No) #REQUIRED>
			&lt;!ATTLIST Grid SizeX CDATA #REQUIRED>
			&lt;!ATTLIST Grid SizeY CDATA #REQUIRED>

			&lt;!--
			 GUI Node, Link and Text elements section
			-->
			&lt;!ELEMENT NodeLinkGUI (GUIFont)>
			&lt;!ATTLIST NodeLinkGUI UniqIDRef IDREF #REQUIRED>
			&lt;!ELEMENT NodesGUI (NodeGUI)*>
			&lt;!ELEMENT NodeGUI (NodeLinkGUI,GUICoord2,DecisionNode?)>
			&lt;!ATTLIST NodeGUI DiagramRefGUI IDREF #IMPLIED>
			&lt;!ELEMENT LinksGUI (LinkGUI)*>
			&lt;!ELEMENT LinkGUI (NodeLinkGUI,GUICoords,BOProbe?,Connect?,DecisionLink?)>
			&lt;!ELEMENT DecisionLink (Label,Desc)>
			&lt;!ATTLIST DecisionLink UniqID ID #REQUIRED>
			&lt;!ELEMENT BOProbe (BOUnit*)>
			&lt;!ATTLIST BOProbe Name CDATA #REQUIRED>
			&lt;!ATTLIST BOProbe LinkID IDREF #REQUIRED>
			&lt;!ATTLIST BOProbe ActionID IDREF #REQUIRED>
			&lt;!ATTLIST BOProbe To IDREF #REQUIRED>
			&lt;!ELEMENT BOUnit (#PCDATA)>
			&lt;!ATTLIST LinkGUI IsOrthogonal (Yes|No) #REQUIRED>
			&lt;!ELEMENT Texts (Text)*>
			&lt;!ELEMENT Text (TextValue,GUIFont,GUICoord2)>
			&lt;!ELEMENT TextValue (#PCDATA)>

			&lt;!--
			 ModelElement
			-->
			&lt;!ELEMENT ModelElement (Desc)>
			&lt;!ATTLIST ModelElement Version CDATA #REQUIRED>

			&lt;!--
			 ModelStructureElement
			-->
			&lt;!ELEMENT ModelStructureElement (ModelElement)>
			&lt;!ATTLIST ModelStructureElement Name CDATA #REQUIRED>

			&lt;!--
			 ModelActivityElement
			-->
			&lt;!ELEMENT ModelActivityElement (ModelElement,Label)>
			&lt;!ATTLIST ModelActivityElement UniqID ID #REQUIRED>
			&lt;!ELEMENT Label (#PCDATA)>

			&lt;!--
			 Misc - dummy elements
			-->
			&lt;!ELEMENT PseudoState (Node)>
			&lt;!ELEMENT ProcessStep (Node)>

			&lt;!--
			 Diagram
			-->
			&lt;!ELEMENT Diagram (ModelStructureElement,Declaration,DiagramBody)>
			&lt;!ATTLIST Diagram Name ID #REQUIRED>
			&lt;!ATTLIST Diagram Version CDATA #REQUIRED>
			&lt;!ELEMENT Declaration (Section1,Section2)>
			&lt;!ELEMENT Section1 (#PCDATA)>
			&lt;!ELEMENT Section2 (#PCDATA)>
			&lt;!ELEMENT DiagramBody (ModelActivityElement,Nodes,Links)>

			&lt;!--
			 Nodes - note that there can be only one start node
			-->
			&lt;!ELEMENT Node (ModelActivityElement)>
			&lt;!ATTLIST Node UserModified (Yes|No) "No">
			&lt;!ELEMENT Nodes ( (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)* |
			 (InitialNode, (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)*) |
			 ( (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)*,InitialNode) |
			 ( (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)*,InitialNode, (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)*))>
			&lt;!ELEMENT CompositeNode (ProcessStep)>
			&lt;!ATTLIST CompositeNode DiagramRef IDREF #REQUIRED>

			&lt;!ELEMENT InitialNode (PseudoState,CorrelationSet*)>
			&lt;!ELEMENT SuccessEndNode (PseudoState)>
			&lt;!ELEMENT FailureEndNode (PseudoState)>
			&lt;!ELEMENT BreakNode (PseudoState)>
			&lt;!ELEMENT ForkNode (PseudoState)>
			&lt;!ELEMENT JoinNode (PseudoState)>
			&lt;!ELEMENT ActionNode (ProcessStep,CodeFragment)>
			&lt;!ELEMENT SubMapNode (ActionNode)>
			&lt;!ATTLIST SubMapNode SubMapName CDATA #REQUIRED>
			&lt;!ATTLIST SubMapNode SubMapType CDATA #REQUIRED>
			&lt;!ELEMENT CodeFragment (#PCDATA)>
			&lt;!ATTLIST CodeFragment Type (Java|XML) #REQUIRED>
			&lt;!ELEMENT ServiceNode (ProcessStep,RegBusObjName,RegBusObjVerb,RegBusObjRefName,CmpBusObjName,CmpBusObjVerb,CmpBusObjRefName,WaitTimeOut,CorrelationSet*)>
			&lt;!ELEMENT RegBusObjName (#PCDATA)>
			&lt;!ELEMENT RegBusObjVerb (#PCDATA)>
			&lt;!ELEMENT RegBusObjRefName (#PCDATA)>
			&lt;!ELEMENT CmpBusObjName (#PCDATA)>
			&lt;!ELEMENT CmpBusObjVerb (#PCDATA)>
			&lt;!ELEMENT CmpBusObjRefName (#PCDATA)>
			&lt;!ELEMENT WaitTimeOut (#PCDATA)>
			&lt;!ATTLIST WaitTimeOut Type (NotUsed|Constant|Parameter|Variable) #REQUIRED>
			&lt;!ELEMENT CorrelationSet (CorrelationAttribute+)>
			&lt;!ATTLIST CorrelationSet IsInit (Yes|No) "No">
			&lt;!ELEMENT CorrelationAttribute (AttributeName,VariableName)>
			&lt;!ELEMENT AttributeName (#PCDATA)>
			&lt;!ELEMENT VariableName (#PCDATA)>
			&lt;!ELEMENT OutboundServiceNode (ServiceNode)>
			&lt;!ATTLIST OutboundServiceNode IsSync CDATA #REQUIRED>
			&lt;!ELEMENT InboundServiceNode (ServiceNode)>
			&lt;!ELEMENT SubDiagramNode (CompositeNode)>
			&lt;!ELEMENT IteratorNode (CompositeNode,IteratorVariable,ThruThisObject,Start,Condition,Increment)>
			&lt;!ATTLIST IteratorNode Type (Attribute|BusObj|Loop) #REQUIRED>
			&lt;!ELEMENT IteratorVariable (#PCDATA)>
			&lt;!ELEMENT ThruThisObject (#PCDATA)>
			&lt;!ELEMENT Start (#PCDATA)>
			&lt;!ELEMENT Condition (#PCDATA)>
			&lt;!ELEMENT Increment (#PCDATA)>
			&lt;!ELEMENT DecisionNode (Label,Desc,DecisionBranches)>
			&lt;!ATTLIST DecisionNode UniqID ID #REQUIRED>
			&lt;!ELEMENT DecisionBranch (Condition,Comment,Label,LinkID)>
			&lt;!ELEMENT DecisionBranches (DecisionBranch,DecisionBranch+)>
			&lt;!ATTLIST DecisionBranch Type (NotUsed|Normal|Default|Exception) #REQUIRED>
			&lt;!ELEMENT LinkID (#PCDATA)>

			&lt;!--
			 Links
			-->
			&lt;!ELEMENT Link (ModelActivityElement,Connect)>
			&lt;!ELEMENT Links (TransitionLink|MessageLink|ExceptionLink)*>
			&lt;!ELEMENT Connect EMPTY>
			&lt;!ATTLIST Connect From IDREF #REQUIRED>
			&lt;!ATTLIST Connect To IDREF #REQUIRED>

			&lt;!ELEMENT TransitionLink (Link,GuardCondition)>
			&lt;!ELEMENT GuardCondition (#PCDATA)>
			&lt;!ELEMENT ExceptionLink (Link,ExceptionName)>
			&lt;!ELEMENT ExceptionName (#PCDATA)>
			&lt;!ELEMENT MessageLink (Link)>

			&lt;!--
			 Misc properties stuff - not exactly belong to diagram, but we stick them here!
			-->
			&lt;!ELEMENT Properties (Property)*>
			&lt;!ELEMENT Property (Attributes)>
			&lt;!ATTLIST Property Name CDATA #REQUIRED>
			&lt;!ELEMENT Attributes (Attribute)*>
			&lt;!ELEMENT Attribute (Value,Desc)>
			&lt;!ATTLIST Attribute Name CDATA #REQUIRED>
			&lt;!ATTLIST Attribute Type CDATA #REQUIRED>

			&lt;!--
			 Messages Element
			-->
			&lt;!ELEMENT Messages (CommonMessages,SpecificMessages)>
			&lt;!ELEMENT CommonMessages (#PCDATA)>
			&lt;!ELEMENT SpecificMessages (Msg)*>
			&lt;!ELEMENT Msg (MsgBody, MsgExpl)>
			&lt;!ATTLIST Msg msg_key CDATA #REQUIRED>
			&lt;!ELEMENT MsgBody (#PCDATA)>
			&lt;!ELEMENT MsgExpl (#PCDATA)>
			&lt;!ATTLIST SpecificMessages locale CDATA #REQUIRED>

			]>
		</xsl:text>	

		<!-- *** Now write the real XML content for the root element CollabTemplate *** -->
		<!-- *** If only the source has some valid XMI Contents otherwise just create an empty template *** -->

		<xsl:variable name="strCollaborationName">
			<xsl:variable name="strModelName" select="normalize-space(XMI/XMI.content/UML:Model/@name)" />
			<xsl:choose>				
				<xsl:when test="not($strModelName = '')">
					<xsl:value-of select="$strModelName" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="'UnknownTemplateName'" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>		
		

<!--		<xsl:value-of select="'test1'" />
			<xsl:for-each select="/XMI/XMI.content/UML:Model/@name">
				<xsl:value-of select="name()" />
			</xsl:for-each>
		<xsl:value-of select="'test1'" / -->
			
		<xsl:element name="CollabTemplate">
			<xsl:attribute name="Name">
				<xsl:value-of select="$strCollaborationName" />				
			</xsl:attribute>
			<xsl:attribute name="Version">
				<xsl:value-of select="$strTemplateVersion" />
			</xsl:attribute>
			<xsl:attribute name="FormatVersion">
				<xsl:value-of select="$strTemplateFormatVersion" />
			</xsl:attribute>
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeHeader" >
				<xsl:with-param name="strTitle" select="'CrossWorlds Collaboration Template'" />
			</xsl:call-template>
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeTemplateDefinition">
				<xsl:with-param name="strCollaborationName" select="$strCollaborationName" />
			</xsl:call-template>
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeGUITemplateDefinition" />
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeScenarios" />
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeMessages" />
			<xsl:value-of select="$newline" />

		</xsl:element>

	</xsl:template>

	<!-- *** create the standard collab template header *** -->
	<xsl:template name="writeHeader">
	<xsl:param name="strTitle" select="'CrossWorlds Collaboration Template'" />
		<Header>
			<Title>
				<xsl:value-of select="$strTitle" />
			</Title>
			<Desc>Automatically Transfomed by IBM CrossWorlds Process Designer converter</Desc>
			<CreateTime>
				<xsl:attribute name="Format">ISO8601GMT</xsl:attribute>
			</CreateTime>
			<ModifyTime>
				<xsl:attribute name="Format">ISO8601GMT</xsl:attribute>
			</ModifyTime>
		</Header>
	</xsl:template>

	<!-- *** create the standard collab template definition *** -->
	<xsl:template name="writeTemplateDefinition">
		<xsl:param name="strCollaborationName" />
		<TemplateDefinition>
			<Desc />
			<Properties>
				<Property>
					<xsl:attribute name="Name">PropertyValues</xsl:attribute>
					<Attributes>
						<Attribute>
							<xsl:attribute name="Name">ClassName</xsl:attribute>
							<xsl:attribute name="Type">0</xsl:attribute>
							<Value>
								<xsl:value-of select="concat('UserCollaborations.', $strCollaborationName)" />
							</Value>
							<Desc>Recovered by UI</Desc>
						</Attribute>
					</Attributes>					
				</Property>
				<Property>
					<xsl:attribute name="Name">SystemProperties</xsl:attribute>
					<Attributes>
						<Attribute>
							<xsl:attribute name="Name">LongLivedBusinessProcess</xsl:attribute>
							<xsl:attribute name="Type">0</xsl:attribute>
							<Value>false</Value>
							<Desc>Is the template long lived</Desc>
						</Attribute>
					</Attributes>					
				</Property>
				<Property>
					<xsl:attribute name="Name">Imports</xsl:attribute>
					<Attributes />
				</Property>
				<Property>
					<xsl:attribute name="Name">Declaration</xsl:attribute>
					<Attributes>
						<Attribute>
							<xsl:attribute name="Name">Entry1</xsl:attribute>
							<xsl:attribute name="Type">0</xsl:attribute>
							<Value></Value>
							<Desc />
						</Attribute>
					</Attributes>					
				</Property>
			</Properties>
			<MinTransactionLevel>
				<xsl:attribute name="Level">None</xsl:attribute>
			</MinTransactionLevel>
			<Package />
			<GeneratedVars />
			<BusObjRefs />
		</TemplateDefinition>					
	</xsl:template>

	<!-- *** create the standard collab template GUI definition *** -->
	<xsl:template name="writeGUITemplateDefinition" >
		<GUITemplateDefinition>
			<GUIProperties>
				<GeneralProperties />
			</GUIProperties>
			<GUIImports />
			<GUIUserSection1 />
			<GUIUserSection2 />
			<GUIUserSection3 />
			<GUIVariables />
		</GUITemplateDefinition>
	</xsl:template>

	<!-- *** create the standard collab template scenarios ** -->
	<xsl:template name="writeScenarios" >
		<xsl:variable name="treeScenarios">

		<!-- *** 
		search for UML:ActivityGraph. 
		ASSUMPTION: Our Scenarios and its diagrams will be based ONLY upon UML:ActivityGraph, no other place as of now. 
	*** -->
	
			<xsl:for-each select="/XMI/XMI.content//UML:Model[*]/UML:Namespace.ownedElement/UML:ActivityGraph" >
				<xsl:variable name="treeActivityGraph" select="." /> 
				<xsl:variable name="nodeActivityGraph" select="xalan:nodeset($treeActivityGraph)" />				

				<!-- *** 
				Now see whether UML:ActivityGraph has any valid diagrams which can be converted to the scenario's main diagram. 
				Validity of the diagram is based on whether it the diagram has atleaset one diagram element in it.
			*** -->			

				<xsl:variable name="ActivityGraphID" select="@xmi.id" />
				<xsl:variable name="strActivityGraphName" select="normalize-space(@name)" />
				<xsl:variable name="nodeActivityGraphCompositeStates" select="UML:StateMachine.top[*]/UML:CompositeState[*]" />


				<xsl:choose>						
					<xsl:when test="count(/XMI/XMI.content/UML:Diagram[@owner= $ActivityGraphID]) > 0">

						<!-- *** 
						Yes, It has valid diagram(s) and now create a scenario for each diagram.				 
					*** -->					

						<xsl:call-template name="writeScenarioElements">
							<xsl:with-param name="nodeActivityGraph" select="$nodeActivityGraph"/>
							<xsl:with-param name="strGivenScenarioName" select="$strActivityGraphName" />
							<xsl:with-param name="DiagramOwnerID" select="$ActivityGraphID" />
							<xsl:with-param name="nodeCompositeStateorSubactivtyState" select="$nodeActivityGraphCompositeStates"/>		
						</xsl:call-template>
						
					</xsl:when>			
					<xsl:otherwise>
						<!-- *** 
						It has no valid diagram(s).
						Now go though its child UML:SubactivityState and see whehter it has any valid diagrams.				 
					*** -->

						<xsl:for-each select="UML:StateMachine.top[*]/UML:CompositeState[*]/UML:CompositeState.subvertex[*]/UML:SubactivityState" >

							<xsl:variable name="treeSubactivtyState" select="." /> 
							<xsl:variable name="nodeSubactivtyState" select="xalan:nodeset($treeSubactivtyState)" />

							<xsl:call-template name="writeScenarioElements">
								<xsl:with-param name="nodeActivityGraph" select="$nodeActivityGraph"/>
								<xsl:with-param name="strGivenScenarioName" select="normalize-space(@name)" />
								<xsl:with-param name="DiagramOwnerID" select="@xmi.id" />
								<xsl:with-param name="nodeCompositeStateorSubactivtyState" select="$nodeSubactivtyState"/>		
							</xsl:call-template>

						</xsl:for-each>
					</xsl:otherwise>
				</xsl:choose>			
			</xsl:for-each>			
		</xsl:variable>

		<xsl:variable name="nodeScenarios" select="xalan:nodeset($treeScenarios)" />

<!--		<OOPs>
		<xsl:copy-of select="$nodeScenarios" />
		</OOPs>
-->

		<xsl:element name="Scenarios">
			<xsl:for-each select="$nodeScenarios/Scenario" >
				<xsl:call-template name="writeScenario" />
			</xsl:for-each>
		</xsl:element>

	</xsl:template>

	<xsl:template name="writeScenario">
		<xsl:element name="{name()}">
			<xsl:call-template name="writeAllAttributes" />
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeHeader" >
				<xsl:with-param name="strTitle" select="'CrossWorlds Process Designer Scenario'" />
			</xsl:call-template>
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeScenarioGUIDefinition" />
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeScenarioGUIDiagrams" />
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeModelScenario" />
			<xsl:value-of select="$newline" />

		</xsl:element>
	</xsl:template>



	<!-- *** create the standard collab template messages ** -->
	<xsl:template name="writeMessages" >
		<Messages>
			<CommonMessages />
			<SpecificMessages>
				<xsl:attribute name="locale">ENU_US</xsl:attribute>
			</SpecificMessages>
		</Messages>
	</xsl:template>

	<xsl:template name="writeScenarioElements">
		<xsl:param name="nodeActivityGraph" />
		<xsl:param name="strGivenScenarioName" />
		<xsl:param name="DiagramOwnerID" />
		<xsl:param name="nodeCompositeStateorSubactivtyState" />		

		<xsl:if test="count(/XMI/XMI.content/UML:Diagram[@owner= $DiagramOwnerID]) > 0">
			<!-- *** 
			Yes, It has valid diagram(s) and now create a scenario for each diagram.				 
		*** -->
			<xsl:for-each select="/XMI/XMI.content/UML:Diagram[@owner= $DiagramOwnerID]" >						
				<xsl:variable name="strScenarioName">
					<xsl:choose>
						<xsl:when test="count(/XMI/XMI.content/UML:Diagram[@owner= $DiagramOwnerID]) > 1">
							<xsl:value-of select="normalize-space(@name)" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$strGivenScenarioName" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
					
				<xsl:element name="Scenario">
					<xsl:attribute name="Name">
						<xsl:value-of select="$strScenarioName" />									
					</xsl:attribute>
					<xsl:attribute name="Version">
						<xsl:value-of select="$strScenarioVersion" />									
					</xsl:attribute>
					<xsl:attribute name="DiagramRef">
						<xsl:value-of select="$strScenarioName" />									
					</xsl:attribute>
					<xsl:attribute name="DiagramRefGUI">
						<xsl:value-of select="concat('GUI__', $strScenarioName)" />									
					</xsl:attribute>									
					<xsl:value-of select="$newline" />											
					<xsl:value-of select="$newline" />

					<xsl:call-template name="writeDiagramElements">
						<xsl:with-param name="nodeActivityGraph" select="$nodeActivityGraph" />
						<xsl:with-param name="nodeParentState" select="$nodeCompositeStateorSubactivtyState" />
						<xsl:with-param name="strDiagramName" select="$strScenarioName" />
						<xsl:with-param name="strModelStructName" select="$strScenarioName" />
					</xsl:call-template>
					<xsl:value-of select="$newline" />
				</xsl:element>
			</xsl:for-each>							
		</xsl:if>
	</xsl:template>

	<xsl:template name="writeDiagramElements">
		<xsl:param name="nodeActivityGraph" />
		<xsl:param name="nodeParentState" />
		<xsl:param name="strDiagramName" />
		<xsl:param name="strModelStructName" />
		<xsl:element name="Diagram">
			<xsl:attribute name="Name">
				<xsl:value-of select ="$strDiagramName" />	
			</xsl:attribute>
			<xsl:attribute name="ModelStructName">
				<xsl:value-of select ="$strModelStructName" />	
			</xsl:attribute>

			<xsl:variable name="treeNodes">
				<xsl:call-template name="writeDiagramElementsNodes">
					<xsl:with-param name="nodeActivityGraph" select="$nodeActivityGraph" />
					<xsl:with-param name="nodeParentState" select="$nodeParentState" />											
					<xsl:with-param name="strDiagramName" select="$strDiagramName" />				
				</xsl:call-template>
			</xsl:variable>
			<xsl:variable name="nodeNodes" select="xalan:nodeset($treeNodes)" />

			<xsl:copy-of select="$nodeNodes" />

			<xsl:element name="Links">
				<xsl:for-each select="UML:Diagram.element/UML:DiagramElement[normalize-space(@style) = 'Transition']">
					<xsl:variable name="SubjectID" select="@subject" />
					<xsl:variable name="DiagramModelTransition" select="$nodeActivityGraph/UML:StateMachine.transitions[*]/UML:Transition[@xmi.id = $SubjectID]" />
					<xsl:if test="boolean($DiagramModelTransition)">

						<xsl:variable name="nodeFrom" select="$nodeNodes/Nodes/*[@xmi.id = $DiagramModelTransition/@source]" />
						<xsl:variable name="nodeTo" select="$nodeNodes/Nodes/*[@xmi.id = $DiagramModelTransition/@target]" />
						<xsl:variable name="strGUIFromUniqueID" select="$nodeNodes/Nodes/*[@xmi.id = $DiagramModelTransition/@source]/@pd.id" />
						<xsl:variable name="strGUIToUniqueID" select="$nodeNodes/Nodes/*[@xmi.id = $DiagramModelTransition/@target]/@pd.id" />

						<xsl:variable name="strServerFromUniqueID">
							<xsl:choose>
								<xsl:when test="($nodeFrom/@pd.node.name = 'DecisionNode')">									
									<xsl:value-of select="$nodeNodes/Nodes/*[@xmi.id = $nodeFrom/pd.decision.source/@xmi.id]/@pd.id" />									
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$nodeFrom/@pd.id" />									
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						
						<xsl:variable name="strServerToUniqueID">
							<xsl:choose>
								<xsl:when test="($nodeTo/@pd.node.name = 'DecisionNode')">									
									<xsl:value-of select="'DONOTWRITE'" />									
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$nodeTo/@pd.id" />									
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						
						<xsl:variable name="strStereoSearch" select="concat(' ', normalize-space($DiagramModelTransition/@xmi.id), ' ')" />

						<xsl:if test="not(normalize-space($strGUIFromUniqueID) = '') and not(normalize-space($strGUIToUniqueID) = '')">
							<xsl:element name="DiagramTransitionElement">
								<xsl:attribute name="xmi.id">
									<xsl:value-of select ="$DiagramModelTransition/@xmi.id" />	
								</xsl:attribute>
								<xsl:attribute name="pd.id">
									<xsl:value-of select ="concat('_', java:getNextID($objXSLTUtils))" />	
								</xsl:attribute>
								<xsl:attribute name="Label">
									<xsl:value-of select ="$DiagramModelTransition/@name" />	
								</xsl:attribute>
								<xsl:attribute name="pd.id.GUIFrom">
									<xsl:value-of select ="$strGUIFromUniqueID" />	
								</xsl:attribute>
								<xsl:attribute name="pd.id.GUITo">
									<xsl:value-of select ="$strGUIToUniqueID" />	
								</xsl:attribute>
								<xsl:attribute name="pd.id.ServerFrom">
									<xsl:value-of select ="$strServerFromUniqueID" />	
								</xsl:attribute>
								<xsl:attribute name="pd.id.ServerTo">
									<xsl:value-of select ="$strServerToUniqueID" />	
								</xsl:attribute>
								<xsl:attribute name="pd.link.name">
									<xsl:variable name="nodeStereotype" select="$nodeActivityGraph/../UML:Stereotype[contains(@extendedElement, $strStereoSearch)]" />
									<xsl:choose>
										<xsl:when test="boolean($nodeStereotype) and ($nodeStereotype/@name = 'Message')">
											<xsl:value-of select="'MessageLink'" />
										</xsl:when>
										<xsl:when test="boolean($nodeStereotype) and ($nodeStereotype/@name = 'Exception')">
											<xsl:value-of select="'ExceptionLink'" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="'TransitionLink'" />
										</xsl:otherwise>
									</xsl:choose>
								</xsl:attribute>
								<xsl:attribute name="pd.link.condition">
									<xsl:value-of select="$DiagramModelTransition/UML:Transition.guard/UML:Guard/UML:Guard.expression/UML:BooleanExpression/@body" />
								</xsl:attribute>

								<xsl:call-template name="CreateGUIFont" />
								<xsl:call-template name="CreateGUICoord" />	
								<Desc>									
									<xsl:value-of select="/XMI/XMI.content//UML:TaggedValue[@tag = 'documentation' and @modelElement = $DiagramModelTransition/@xmi.id]/@value" />
								</Desc>
															
							</xsl:element>
						</xsl:if>
					</xsl:if>														
				</xsl:for-each>									
			</xsl:element>

			<xsl:element name="Texts">
			</xsl:element>

		</xsl:element>
	</xsl:template>

	<xsl:template name="writeDiagramElementsNodes">
		<xsl:param name="nodeActivityGraph" />
		<xsl:param name="nodeParentState" />
		<xsl:param name="strDiagramName" />

		<xsl:element name="Nodes" >
			<xsl:for-each select="UML:Diagram.element/UML:DiagramElement[not(normalize-space(@style) = 'Transition')]">
				<xsl:variable name="SubjectID" select="@subject" />								
				<xsl:variable name="DiagramModelElement" select="$nodeParentState[*]/UML:CompositeState.subvertex[*]/*[@xmi.id = $SubjectID]" />

				<xsl:choose>
					<xsl:when test="boolean($DiagramModelElement)">
						<xsl:variable name="strUniqueId" select="concat('_', java:getNextID($objXSLTUtils))" />						
						<xsl:variable name="strStereoSearch" select="concat(' ', normalize-space($DiagramModelElement/@xmi.id), ' ')" />					

						<xsl:element name="DiagramNodeElement">
							<xsl:attribute name="xmi.id">
								<xsl:value-of select ="$DiagramModelElement/@xmi.id" />	
							</xsl:attribute>
							<xsl:attribute name="pd.id">
								<xsl:value-of select ="$strUniqueId" />	
							</xsl:attribute>
							<xsl:attribute name="Label">
								<xsl:value-of select ="$DiagramModelElement/@name" />	
							</xsl:attribute>
							<xsl:attribute name="pd.node.name">
								<xsl:choose>
									<xsl:when test="name($DiagramModelElement) = 'UML:Pseudostate' and $DiagramModelElement/@kind = 'initial'">
										<xsl:value-of select="'InitialNode'" />
									</xsl:when>
									<xsl:when test="name($DiagramModelElement) = 'UML:Pseudostate' and $DiagramModelElement/@kind = 'branch'">
										<xsl:value-of select="'DecisionNode'" />
									</xsl:when>
									<xsl:when test="name($DiagramModelElement) = 'UML:ActionState'">
										<xsl:variable name="nodeStereotype" select="$nodeActivityGraph/../UML:Stereotype[contains(@extendedElement, $strStereoSearch)]" />
										<xsl:choose>
											<xsl:when test="boolean($nodeStereotype) and ($nodeStereotype/@name = 'SyncOutboundService')">
												<xsl:value-of select="'SyncOutboundService'" />
											</xsl:when>
											<xsl:when test="boolean($nodeStereotype) and ($nodeStereotype/@name = 'ASyncOutboundService')">
												<xsl:value-of select="'ASyncOutboundService'" />
											</xsl:when>
											<xsl:when test="boolean($nodeStereotype) and ($nodeStereotype/@name = 'InboundService')">
												<xsl:value-of select="'InboundService'" />
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="'ActionNode'" />
											</xsl:otherwise>
										</xsl:choose>
									</xsl:when>
									<xsl:when test="name($DiagramModelElement) = 'UML:SubactivityState'">
										<xsl:variable name="nodeStereotype" select="$nodeActivityGraph/../UML:Stereotype[contains(@extendedElement, $strStereoSearch)]" />
										<xsl:choose>
											<xsl:when test="boolean($nodeStereotype) and ($nodeStereotype/@name = 'Iterator')">
												<xsl:value-of select="'IteratorNode'" />
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="'SubDiagramNode'" />
											</xsl:otherwise>
										</xsl:choose>
									</xsl:when>
									<xsl:when test="name($DiagramModelElement) = 'UML:FinalState' and ($DiagramModelElement/@xmi.label = 'SuccessEnd')">
										<xsl:value-of select="'SuccessEndNode'" />
									</xsl:when>
									<xsl:when test="name($DiagramModelElement) = 'UML:FinalState' and ($DiagramModelElement/@xmi.label = 'FailureEnd')">
										<xsl:value-of select="'FailureEndNode'" />
									</xsl:when>
									<xsl:when test="name($DiagramModelElement) = 'UML:FinalState' and ($DiagramModelElement/@xmi.label = 'Break')">
										<xsl:value-of select="'BreakNode'" />
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="'ActionNode'" />									
									</xsl:otherwise>									
								</xsl:choose>							
							</xsl:attribute>


							<xsl:call-template name="CreateGUIFont">
								<xsl:with-param name="strStyle" select="normalize-space(@style)" />
							</xsl:call-template>

							<xsl:call-template name="CreateGUICoord" />
							<Desc>									
								<xsl:value-of select="/XMI/XMI.content//UML:TaggedValue[@tag = 'documentation' and @modelElement = $DiagramModelElement/@xmi.id]/@value" />
							</Desc>


							<xsl:if test="name($DiagramModelElement) = 'UML:Pseudostate' and $DiagramModelElement/@kind = 'branch'">
							<!-- now find other information related to the decision node -->
								<xsl:variable name="strThisXMIID" select="$DiagramModelElement/@xmi.id" />
								<xsl:element name="pd.decision.source">
									<xsl:attribute name="xmi.id">
										<xsl:value-of select="$nodeActivityGraph/UML:StateMachine.transitions[*]/UML:Transition[@target = $strThisXMIID]/@source" />
									</xsl:attribute>									
								</xsl:element>
								<pd.decision.branches>
									<xsl:for-each select="$nodeActivityGraph/UML:StateMachine.transitions[*]/UML:Transition[@source = $strThisXMIID]">									
										<xsl:element name="pd.decision.branch">
											<xsl:attribute name="xmi.id">
												<xsl:value-of select="@xmi.id" />
											</xsl:attribute>

											<xsl:variable name="strStereoSearch" select="concat(' ', normalize-space(@xmi.id), ' ')" />
											<xsl:variable name="nodeStereotype" select="$nodeActivityGraph/../UML:Stereotype[contains(@extendedElement, $strStereoSearch)]" />
											<xsl:variable name="strTransitionXMIID" select="@xmi.id" />
											<type>
												<xsl:choose>
													<xsl:when test="boolean($nodeStereotype) and ($nodeStereotype/@name = 'Exception')">
														<xsl:value-of select="'Exception'" />
													</xsl:when>
													<xsl:otherwise>
														<xsl:value-of select="'Normal'" />
													</xsl:otherwise>
												</xsl:choose>
											</type>
											<condition>
												<xsl:value-of select="UML:Transition.guard/UML:Guard/UML:Guard.expression/UML:BooleanExpression/@body" />												
											</condition>
											<label>
												<xsl:value-of select ="@name" />													
											</label>
											<comment>
												<xsl:value-of select="/XMI/XMI.content//UML:TaggedValue[@tag = 'documentation' and @modelElement = $strTransitionXMIID]/@value" />											
											</comment>
										</xsl:element>
									</xsl:for-each>
								</pd.decision.branches>
							</xsl:if>
							
							<xsl:choose>															
								<xsl:when test="name($DiagramModelElement) = 'UML:ActionState'">
									<xsl:variable name="nodeActionExpression" select="$DiagramModelElement/UML:State.doActivity/UML:ActionSequence/UML:ActionSequence.action/UML:UninterpretedAction/UML:Action.script/UML:ActionExpression" />								
									<xsl:variable name="strCodeType">
										<xsl:choose>
											<xsl:when test="$nodeActionExpression/@language = 'XML'">
												<xsl:value-of select="'XML'" />
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="'Java'" />
											</xsl:otherwise>
										</xsl:choose>
									</xsl:variable>
										
									<xsl:element name="pd.codefragment">
										<xsl:attribute name="type">
											<xsl:value-of select ="$strCodeType" />	
										</xsl:attribute>
										<xsl:value-of select ="$nodeActionExpression/UML:Expression.body" />																			
									</xsl:element>								
								</xsl:when>	
								<xsl:otherwise>						
									<xsl:element name="pd.codefragment">
										<xsl:attribute name="type">
											<xsl:value-of select ="'XML'" />	
										</xsl:attribute>
										<xsl:value-of select ="''" />																			
									</xsl:element>								
								</xsl:otherwise>
							</xsl:choose>

							<xsl:if test="name($DiagramModelElement) = 'UML:SubactivityState'">

								<xsl:variable name="nodeStereotype" select="$nodeActivityGraph/../UML:Stereotype[contains(@extendedElement, $strStereoSearch)]" />
								<xsl:variable name="strNodeDiagramType">
									<xsl:choose>
										<xsl:when test="boolean($nodeStereotype) and ($nodeStereotype/@name = 'Iterator')">
											<xsl:value-of select="'Iterator'" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="'SubDiagram'" />
										</xsl:otherwise>
									</xsl:choose>
								</xsl:variable>

								<xsl:variable name="strSubDiagramName" select="concat($strDiagramName, '.', $strNodeDiagramType, $strUniqueId)" />
								<xsl:element name="SubDiagramRef">
									<xsl:value-of select="$strSubDiagramName" />
								</xsl:element>

								<xsl:variable name="SubactivityStateModelID" select="$DiagramModelElement/@xmi.id" />
								<xsl:if test="count(/XMI/XMI.content/UML:Diagram[@owner= $SubactivityStateModelID]) > 0">
										<!-- *** 
										Yes, It has valid diagram(s) and now create a scenario for each diagram.				 
									*** -->
									<xsl:for-each select="/XMI/XMI.content/UML:Diagram[@owner= $SubactivityStateModelID]" >						

										<xsl:call-template name="writeDiagramElements">
											<xsl:with-param name="nodeActivityGraph" select="$nodeActivityGraph" />
											<xsl:with-param name="nodeParentState" select="$DiagramModelElement" />											
											<xsl:with-param name="strDiagramName" select="$strSubDiagramName" />
											<xsl:with-param name="strModelStructName" select="concat($strNodeDiagramType , $strUniqueId)" />											
										</xsl:call-template>
									</xsl:for-each>								
								</xsl:if>
							</xsl:if>
						</xsl:element>
					</xsl:when>
					<xsl:otherwise>
						<!-- may be we need to search for this in other places -->
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>	
		</xsl:element>
	</xsl:template>			
	
	<xsl:template name="calculateDiagramRegion">
		<xsl:param name="nMinX" select="0" />
		<xsl:param name="nMaxX" select="0" />
		<xsl:param name="nMinY" select="0" />
		<xsl:param name="nMaxY" select="0" />		
		<xsl:param name="nSizeMinY" select="0" />
		<xsl:param name="nNodeIndex" select="1" />
		<xsl:param name="nNodesCount" />		
		<xsl:param name="nodeCollection" />	
		<xsl:param name="nMinXNodeID" select="'-1'" />
		<xsl:param name="nMinYNodeID" select="'-1'" />
		
		<xsl:choose>
			<xsl:when test="$nNodeIndex = 1 and ($nNodeIndex &lt;= $nNodesCount) and $nMinX = 0 and $nMinY = 0">
				<xsl:call-template name="calculateDiagramRegion">		
					<xsl:with-param name="nMinX" select="$nodeCollection/*[$nNodeIndex]/GUICoord[1]/@X" />
					<xsl:with-param name="nMaxX" select="$nodeCollection/*[$nNodeIndex]/GUICoord[2]/@X" />
					<xsl:with-param name="nMinY" select="$nodeCollection/*[$nNodeIndex]/GUICoord[1]/@Y" />
					<xsl:with-param name="nMaxY" select="$nodeCollection/*[$nNodeIndex]/GUICoord[2]/@Y" />
					<xsl:with-param name="nMinXNodeID" select="$nodeCollection/*[$nNodeIndex]/@pd.id" />
					<xsl:with-param name="nMinYNodeID" select="$nodeCollection/*[$nNodeIndex]/@pd.id" />
					<xsl:with-param name="nNodeIndex" select="$nNodeIndex + 1" />						
					<xsl:with-param name="nNodesCount" select="$nNodesCount" />
					<xsl:with-param name="nodeCollection" select="$nodeCollection" />
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="$nNodeIndex &lt;= $nNodesCount">
			
			
				<xsl:variable name="treeCurrentRegion">
					<xsl:element name="CurrentRegion" >
						<xsl:element name="pd.min.x">
							<xsl:choose>
								<xsl:when test="$nMinX &gt; $nodeCollection/*[$nNodeIndex]/GUICoord[1]/@X">
									<xsl:attribute name="pd.node.id">
										<xsl:value-of select="$nodeCollection/*[$nNodeIndex]/@pd.id" />
									</xsl:attribute>
									<xsl:value-of select="$nodeCollection/*[$nNodeIndex]/GUICoord[1]/@X" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:attribute name="pd.node.id">
										<xsl:value-of select="$nMinXNodeID" />
									</xsl:attribute>
									<xsl:value-of select="$nMinX" />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:element>	
						<xsl:element name="pd.max.x">
							<xsl:choose>
								<xsl:when test="$nMaxX &lt; $nodeCollection/*[$nNodeIndex]/GUICoord[2]/@X">
									<xsl:value-of select="$nodeCollection/*[$nNodeIndex]/GUICoord[2]/@X" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$nMaxX" />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:element>		
						<xsl:element name="pd.min.y">
							<xsl:choose>
								<xsl:when test="$nMinY &gt; $nodeCollection/*[$nNodeIndex]/GUICoord[1]/@Y">
									<xsl:attribute name="pd.node.id">
										<xsl:value-of select="$nodeCollection/*[$nNodeIndex]/@pd.id" />
									</xsl:attribute>
									<xsl:value-of select="$nodeCollection/*[$nNodeIndex]/GUICoord[1]/@Y" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:attribute name="pd.node.id">
										<xsl:value-of select="$nMinYNodeID" />
									</xsl:attribute>
									<xsl:value-of select="$nMinY" />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:element>				
						<xsl:element name="pd.max.y">
							<xsl:choose>
								<xsl:when test="$nMaxY &lt; $nodeCollection/*[$nNodeIndex]/GUICoord[2]/@Y">
									<xsl:value-of select="$nodeCollection/*[$nNodeIndex]/GUICoord[2]/@Y" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$nMaxY" />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:element>
					</xsl:element> 
				</xsl:variable>
				<xsl:variable name="NodeCurrentRegion" select="xalan:nodeset($treeCurrentRegion)" />				

				<xsl:call-template name="calculateDiagramRegion">		
					<xsl:with-param name="nMinX" select="$NodeCurrentRegion/CurrentRegion/pd.min.x" />
					<xsl:with-param name="nMaxX" select="$NodeCurrentRegion/CurrentRegion/pd.max.x" />
					<xsl:with-param name="nMinY" select="$NodeCurrentRegion/CurrentRegion/pd.min.y" />
					<xsl:with-param name="nMaxY" select="$NodeCurrentRegion/CurrentRegion/pd.max.y" />
					<xsl:with-param name="nNodeIndex" select="$nNodeIndex + 1" />						
					<xsl:with-param name="nNodesCount" select="$nNodesCount" />
					<xsl:with-param name="nodeCollection" select="$nodeCollection" />
					<xsl:with-param name="nMinXNodeID" select="$NodeCurrentRegion/CurrentRegion/pd.min.x/@pd.node.id" />
					<xsl:with-param name="nMinYNodeID" select="$NodeCurrentRegion/CurrentRegion/pd.min.y/@pd.node.id" />
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="name($nodeCollection) = 'Nodes'">
						<xsl:value-of select="'Nodes'" />						
						<xsl:call-template name="calculateDiagramRegion">		
							<xsl:with-param name="nMinX" select="$nMinX" />
							<xsl:with-param name="nMaxX" select="$nMaxX" />
							<xsl:with-param name="nMinY" select="$nMinY" />
							<xsl:with-param name="nMaxY" select="$nMaxY" />							
							<xsl:with-param name="nNodesCount" select="count(Links/*)" />
							<xsl:with-param name="nodeCollection" select="./Links" />
							<xsl:with-param name="nMinXNodeID" select="$nMinXNodeID" />
							<xsl:with-param name="nMinYNodeID" select="$nMinYNodeID" />
						</xsl:call-template>
					</xsl:when>				
					<xsl:when test="name($nodeCollection) = 'Links'">						
						<xsl:value-of select="'Links'" />						
						<xsl:call-template name="calculateDiagramRegion">	
															<xsl:with-param name="nMinX" select="$nMinX" />
							<xsl:with-param name="nMaxX" select="$nMaxX" />
							<xsl:with-param name="nMinY" select="$nMinY" />
							<xsl:with-param name="nMaxY" select="$nMaxY" />							
							<xsl:with-param name="nNodesCount" select="count(Texts/*)" />
							<xsl:with-param name="nodeCollection" select="./Texts" />
							<xsl:with-param name="nMinXNodeID" select="$nMinXNodeID" />
							<xsl:with-param name="nMinYNodeID" select="$nMinYNodeID" />
						</xsl:call-template>
					</xsl:when>				
					<xsl:otherwise>
						<pd.min.x>
							<xsl:attribute name="pd.node.id">
								<xsl:value-of select="$nMinXNodeID" />
							</xsl:attribute>
							<xsl:value-of select="$nMinX" />
						</pd.min.x>
						<pd.max.x>
							<xsl:value-of select="$nMaxX" />
						</pd.max.x>
						<pd.min.y>
							<xsl:attribute name="pd.node.id">
								<xsl:value-of select="$nMinYNodeID" />
							</xsl:attribute>
							<xsl:value-of select="$nMinY" />
						</pd.min.y>
						<pd.max.y>
							<xsl:value-of select="$nMaxY" />
						</pd.max.y>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>					


	<!-- *** copies all source element's attribute name and value pairs to the destination element *** -->
	<xsl:template name="writeAllAttributes">
		<xsl:for-each select="@*">	
			<xsl:attribute name="{name()}">
				<xsl:value-of select="." />
			</xsl:attribute>
		</xsl:for-each>
	</xsl:template>

	<!-- *** create the standard Scenario GUI definition *** -->
	<xsl:template name="writeScenarioGUIDefinition" >
		<GUIDefinition>
			<Desc />
			<GUIUserSection1 />
			<GUIUserSection2 />
			<GUIVariables />
		</GUIDefinition>
	</xsl:template>

	<!-- *** write the GUIDiagrams section of Scenario *** -->
	<xsl:template name="writeScenarioGUIDiagrams" >
		<xsl:element name="GUIDiagrams">
			<xsl:for-each select=".//Diagram">
				<xsl:call-template name="writeGUIDiagram" />				
			</xsl:for-each>
		</xsl:element>
	</xsl:template>

	<!-- ***  write the ModelScenario of the Scenario*** -->
	<xsl:template name="writeModelScenario" >
		<xsl:element name="ModelScenario">
			<xsl:for-each select=".//Diagram">
				<xsl:call-template name="writeDiagram" />				
			</xsl:for-each>
		</xsl:element>
	</xsl:template>

	<!-- *** write the Diagram section of  ModelScenario *** -->
	<xsl:template name="writeDiagram" >
		<xsl:element name="Diagram">
			<xsl:attribute name="Name">
				<xsl:value-of select="@Name" />
			</xsl:attribute>
			<xsl:attribute name="Version">
				<xsl:value-of select="'1.0.0'" />
			</xsl:attribute>

			<xsl:element name="ModelStructureElement">
				<xsl:attribute name="Name">
					<xsl:value-of select="@ModelStructName" />
				</xsl:attribute>
				<xsl:element name="ModelElement"> 
					<xsl:attribute name="Version">
						<xsl:value-of select="'1'" />
					</xsl:attribute>

					<Desc />
				</xsl:element>				
			</xsl:element>

			<xsl:element name="Declaration">
				<Section1 />
				<Section2 />
			</xsl:element>

			<xsl:element name="DiagramBody">

				<xsl:variable name="strModelActivityName">
					<xsl:choose>
						<xsl:when test="contains(@ModelStructName, 'SubDiagram_') and not(contains(@ModelStructName, '.')) and contains(@Name, '.')">
							<xsl:value-of select="@ModelStructName" />							
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="concat(@ModelStructName, '_-1')" />							
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<xsl:call-template name="writeModelActivityElement">
					<xsl:with-param name="UniqueID" select="$strModelActivityName" />
				</xsl:call-template>
				<xsl:value-of select="$newline" />


				<xsl:element name="Nodes">
					<xsl:for-each select="Nodes/*">
						<xsl:call-template name="writeNode" />
					</xsl:for-each>				
				</xsl:element>

				<xsl:element name="Links">
					<xsl:for-each select="Links/DiagramTransitionElement">
						<xsl:call-template name="writeLink" />
					</xsl:for-each>				
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<!-- *** write the Node element of Nodes *** -->
	<xsl:template name="writeNode" >
		<xsl:choose>
			<xsl:when test="@pd.node.name ='InitialNode'">
				<xsl:call-template name="writeInitialNode" />				
			</xsl:when>
			<xsl:when test="@pd.node.name ='ActionNode'">
				<xsl:call-template name="writeActionNode" />				
			</xsl:when>
			<xsl:when test="@pd.node.name ='DecisionNode'">
				<!-- Skip It -->				
			</xsl:when>
			<xsl:when test="@pd.node.name = 'SyncOutboundService'">
				<xsl:call-template name="writeSyncOutboundService" />				
			</xsl:when>
			<xsl:when test="@pd.node.name = 'ASyncOutboundService'">
				<xsl:call-template name="writeASyncOutboundService" />				
			</xsl:when>
			<xsl:when test="@pd.node.name = 'InboundService'">
				<xsl:call-template name="writeInboundService" />				
			</xsl:when>
			<xsl:when test="@pd.node.name = 'SubDiagramNode'">
				<xsl:call-template name="writeSubDiagramNode" />				
			</xsl:when>
			<xsl:when test="@pd.node.name = 'IteratorNode'">
				<xsl:call-template name="writeIteratorNode" />				
			</xsl:when>
			<xsl:when test="@pd.node.name = 'SuccessEndNode'">
				<xsl:call-template name="writeSuccessEndNode" />				
			</xsl:when>
			<xsl:when test="@pd.node.name = 'FailureEndNode'">
				<xsl:call-template name="writeFailureEndNode" />				
			</xsl:when>
			<xsl:when test="@pd.node.name = 'BreakNode'">
				<xsl:call-template name="writeBreakNode" />				
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="writeActionNode" />				
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="writeInitialNode">
		<xsl:element name="InitialNode">
			<xsl:element name="PseudoState">
				<xsl:call-template name="writeChildNode" />
			</xsl:element>			
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeActionNode">
		<xsl:element name="ActionNode">
			<xsl:element name="ProcessStep">
				<xsl:call-template name="writeChildNode" />
			</xsl:element>
			
			<xsl:element name="CodeFragment">	
				<xsl:attribute name="Type">
					<xsl:value-of select="pd.codefragment/@type" />
				</xsl:attribute>
				<xsl:value-of select="pd.codefragment" />
			</xsl:element>						
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeSyncOutboundService">
		<xsl:element name="OutboundServiceNode">
			<xsl:attribute name="IsSync">
				<xsl:value-of select="'true'" />
			</xsl:attribute>

			<ServiceNode>
				<xsl:element name="ProcessStep">
					<xsl:call-template name="writeChildNode" />
				</xsl:element>

				<RegBusObjName /> 
				<RegBusObjVerb /> 
				<RegBusObjRefName /> 
				<CmpBusObjName /> 
				<CmpBusObjVerb /> 
				<CmpBusObjRefName /> 
				<WaitTimeOut Type="NotUsed">0</WaitTimeOut> 
			</ServiceNode>			
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeASyncOutboundService">
		<xsl:element name="OutboundServiceNode">
			<xsl:attribute name="IsSync">
				<xsl:value-of select="'false'" />
			</xsl:attribute>

			<ServiceNode>
				<xsl:element name="ProcessStep">
					<xsl:call-template name="writeChildNode" />
				</xsl:element>

				<RegBusObjName /> 
				<RegBusObjVerb /> 
				<RegBusObjRefName /> 
				<CmpBusObjName /> 
				<CmpBusObjVerb /> 
				<CmpBusObjRefName /> 
				<WaitTimeOut Type="NotUsed">0</WaitTimeOut> 
			</ServiceNode>			
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeInboundService">
		<xsl:element name="InboundServiceNode">
			<ServiceNode>
				<xsl:element name="ProcessStep">
					<xsl:call-template name="writeChildNode" />
				</xsl:element>

				<RegBusObjName /> 
				<RegBusObjVerb /> 
				<RegBusObjRefName /> 
				<CmpBusObjName /> 
				<CmpBusObjVerb /> 
				<CmpBusObjRefName /> 
				<WaitTimeOut Type="NotUsed">0</WaitTimeOut> 
				<CorrelationSet IsInit="No">
					<CorrelationAttribute>
						<AttributeName />
						<VariableName />
					</CorrelationAttribute>
				</CorrelationSet>
			</ServiceNode>			
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeSubDiagramNode">
		<xsl:element name="SubDiagramNode">
			<xsl:element name="CompositeNode">
				<xsl:attribute name="DiagramRef">
					<xsl:value-of select="SubDiagramRef" />
				</xsl:attribute>

				<xsl:element name="ProcessStep">
					<xsl:call-template name="writeChildNode" />
				</xsl:element>				
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeIteratorNode">
		<xsl:element name="IteratorNode">
			<xsl:attribute name="Type">
				<xsl:value-of select="'Attribute'" />
			</xsl:attribute>

			<xsl:element name="CompositeNode">
				<xsl:attribute name="DiagramRef">
					<xsl:value-of select="SubDiagramRef" />
				</xsl:attribute>

				<xsl:element name="ProcessStep">
					<xsl:call-template name="writeChildNode" />
				</xsl:element>				
			</xsl:element>
			<IteratorVariable />
			<ThruThisObject />
			<Start />
			<Condition />
			<Increment />
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeSuccessEndNode">
		<xsl:element name="SuccessEndNode">
			<xsl:element name="PseudoState">
				<xsl:call-template name="writeChildNode" />
			</xsl:element>			
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeFailureEndNode">
		<xsl:element name="FailureEndNode">
			<xsl:element name="PseudoState">
				<xsl:call-template name="writeChildNode" />
			</xsl:element>			
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeBreakNode">
		<xsl:element name="BreakNode">
			<xsl:element name="PseudoState">
				<xsl:call-template name="writeChildNode" />
			</xsl:element>			
		</xsl:element>
	</xsl:template>

	<!-- *** write the Node element of ProcessStep and PseudoState *** -->
	<xsl:template name="writeChildNode" >
		<xsl:element name="Node">
			<xsl:attribute name="UserModified">
				<xsl:value-of select="'No'" />
			</xsl:attribute>
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeModelActivityElement">
				<xsl:with-param name="UniqueID" select="@pd.id" />
			</xsl:call-template>
			<xsl:value-of select="$newline" />
		</xsl:element>
	</xsl:template>


	<!-- *** write the TransitionLink element of Links *** -->
	<xsl:template name="writeLink" >
		<xsl:choose>
			<xsl:when test="@pd.id.ServerTo = 'DONOTWRITE'">				
			</xsl:when>
			<xsl:when test="@pd.link.name ='ExceptionLink'">
				<xsl:call-template name="writeExceptionLink" />				
			</xsl:when>
			<xsl:when test="@pd.link.name ='TransitionLink'">
				<xsl:call-template name="writeTransitionLink" />				
			</xsl:when>
			<xsl:when test="@pd.link.name = 'MessageLink'">
				<xsl:call-template name="writeMessageLink" />				
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="writeTransitionLink" >
		<xsl:element name="TransitionLink">
			<xsl:call-template name="writeChildLink" />
			<xsl:value-of select="$newline" />			
						
			<xsl:element name="GuardCondition" >
				<xsl:value-of select="@pd.link.condition" />
			</xsl:element>
			<xsl:value-of select="$newline" />			
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeMessageLink" >
		<xsl:element name="MessageLink">
			<xsl:call-template name="writeChildLink" />
			<xsl:value-of select="$newline" />									
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeExceptionLink" >
		<xsl:element name="ExceptionLink">
			<xsl:call-template name="writeChildLink" />
			<xsl:value-of select="$newline" />			
						
			<xsl:element name="ExceptionName" >
				<xsl:value-of select="@pd.link.condition" />
			</xsl:element>
			<xsl:value-of select="$newline" />			
		</xsl:element>
	</xsl:template>

	<!-- *** write the Link element of TransitionLink *** -->
	<xsl:template name="writeChildLink" >
		<xsl:element name="Link">
			<xsl:call-template name="writeModelActivityElement">
				<xsl:with-param name="UniqueID" select="@pd.id" />
			</xsl:call-template>
						
			<xsl:element name="Connect" >
				<xsl:attribute name="To">
					<xsl:value-of select="@pd.id.ServerTo" />
				</xsl:attribute>
				<xsl:attribute name="From">
					<xsl:value-of select="@pd.id.ServerFrom" />
				</xsl:attribute>
			</xsl:element>
			<xsl:value-of select="$newline" />			
		</xsl:element>
	</xsl:template>


	<!-- *** write the GUIDiagram section of  GUIDiagrams *** -->
	<xsl:template name="writeGUIDiagram" >
		<xsl:element name="GUIDiagram">
			<xsl:attribute name="Name">
				<xsl:value-of select="concat('GUI__', @Name)" />
			</xsl:attribute>
			<xsl:attribute name="Version">
				<xsl:value-of select="'1.0.0'" />
			</xsl:attribute>

			<xsl:call-template name="writeHeader" >
				<xsl:with-param name="strTitle" select="'CrossWorlds Process Designer Diagram'" />
			</xsl:call-template>
			<xsl:value-of select="$newline" />			

			<GUIProperties />
			<xsl:value-of select="$newline" />			

			<xsl:variable name="treeDiagramRegion">			
				<xsl:element name="DiagramRegion">
					<xsl:call-template name="calculateDiagramRegion">				
						<xsl:with-param name="nNodesCount" select="count(Nodes/*)" />
						<xsl:with-param name="nodeCollection" select="./Nodes" />
					</xsl:call-template> 
				</xsl:element>
			</xsl:variable>

			<xsl:variable name="NodeDiagramRegion" select="xalan:nodeset($treeDiagramRegion)" />				

			<xsl:variable name="nDiagramMinXWidth">
				<xsl:variable name="nodeMinXNode" select="Nodes/*[@pd.id = $NodeDiagramRegion/DiagramRegion/pd.min.x/@pd.node.id]" />
				<xsl:variable name="nodeMinXLink" select="Links/*[@pd.id = $NodeDiagramRegion/DiagramRegion/pd.min.x/@pd.node.id]" />
				<xsl:variable name="nodeMinXText" select="Texts/*[@pd.id = $NodeDiagramRegion/DiagramRegion/pd.min.x/@pd.node.id]" />
				<xsl:choose>
					<xsl:when test="not($nodeMinXNode = '')">
						<xsl:variable name="treeSizeScaleFactor">					
							<xsl:choose>
								<xsl:when test="$nodeMinXNode/@pd.node.name = 'InitialNode' or $nodeMinXNode/@pd.node.name = 'SuccessEndNode' or $nodeMinXNode/@pd.node.name = 'FailureEndNode' or $nodeMinXNode/@pd.node.name = 'BreakNode'">
									<xsl:element name="pd.size.scale.x">3</xsl:element>
									<xsl:element name="pd.size.scale.y">2.75</xsl:element>
								</xsl:when>
								<xsl:when test="$nodeMinXNode/@pd.node.name = 'OutboundServiceNode' or $nodeMinXNode/@pd.node.name = 'InboundServiceNode'">
									<xsl:element name="pd.size.scale.x">10</xsl:element>
									<xsl:element name="pd.size.scale.y">3</xsl:element>
								</xsl:when>
								<xsl:when test="$nodeMinXNode/@pd.node.name = 'IteratorNode' or $nodeMinXNode/@pd.node.name = 'SubDiagramNode'">
									<xsl:element name="pd.size.scale.x">5</xsl:element>
									<xsl:element name="pd.size.scale.y">2</xsl:element>
								</xsl:when>
								<xsl:otherwise>
									<xsl:element name="pd.size.scale.x">5</xsl:element>
									<xsl:element name="pd.size.scale.y">2</xsl:element>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:variable name="nodeSizeScaleFactor" select="xalan:nodeset($treeSizeScaleFactor)" />

						<xsl:value-of select="round($nodeMinXNode/GUICoord[2]/@X div ($nodeSizeScaleFactor/pd.size.scale.x * 2))" />
						
					</xsl:when>
					<xsl:when test="not($nodeMinXLink = '')">
						<xsl:variable name="treeSizeScaleFactor">					
							<xsl:element name="pd.size.scale.x">5</xsl:element>
							<xsl:element name="pd.size.scale.y">2</xsl:element>
						</xsl:variable>
						<xsl:variable name="nodeSizeScaleFactor" select="xalan:nodeset($treeSizeScaleFactor)" />
						<xsl:value-of select="round($nodeMinXLink/GUICoord[2]/@X div ($nodeSizeScaleFactor/pd.size.scale.x * 2))" />
					</xsl:when>
					<xsl:when test="not($nodeMinXText = '')">
						<xsl:variable name="treeSizeScaleFactor">					
							<xsl:element name="pd.size.scale.x">5</xsl:element>
							<xsl:element name="pd.size.scale.y">2</xsl:element>							
						</xsl:variable>
						<xsl:variable name="nodeSizeScaleFactor" select="xalan:nodeset($treeSizeScaleFactor)" />
						<xsl:value-of select="round($nodeMinXText/GUICoord[2]/@X div ($nodeSizeScaleFactor/pd.size.scale.x * 2))" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="'10'" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>

			<xsl:variable name="nDiagramMinYWidth">
				<xsl:variable name="nodeMinYNode" select="Nodes/*[@pd.id = $NodeDiagramRegion/DiagramRegion/pd.min.y/@pd.node.id]" />
				<xsl:variable name="nodeMinYLink" select="Links/*[@pd.id = $NodeDiagramRegion/DiagramRegion/pd.min.y/@pd.node.id]" />
				<xsl:variable name="nodeMinYText" select="Texts/*[@pd.id = $NodeDiagramRegion/DiagramRegion/pd.min.y/@pd.node.id]" />
				<xsl:choose>
					<xsl:when test="not($nodeMinYNode = '')">
						<xsl:variable name="treeSizeScaleFactor">					
							<xsl:choose>
								<xsl:when test="$nodeMinYNode/@pd.node.name = 'InitialNode' or $nodeMinYNode/@pd.node.name = 'SuccessEndNode' or $nodeMinYNode/@pd.node.name = 'FailureEndNode' or $nodeMinYNode/@pd.node.name = 'BreakNode'">
									<xsl:element name="pd.size.scale.x">3</xsl:element>
									<xsl:element name="pd.size.scale.y">2.75</xsl:element>
								</xsl:when>
								<xsl:when test="$nodeMinYNode/@pd.node.name = 'OutboundServiceNode' or $nodeMinYNode/@pd.node.name = 'InboundServiceNode'">
									<xsl:element name="pd.size.scale.x">10</xsl:element>
									<xsl:element name="pd.size.scale.y">3</xsl:element>
								</xsl:when>
								<xsl:when test="$nodeMinYNode/@pd.node.name = 'IteratorNode' or $nodeMinYNode/@pd.node.name = 'SubDiagramNode'">
									<xsl:element name="pd.size.scale.x">5</xsl:element>
									<xsl:element name="pd.size.scale.y">2</xsl:element>
								</xsl:when>
								<xsl:otherwise>
									<xsl:element name="pd.size.scale.x">5</xsl:element>
									<xsl:element name="pd.size.scale.y">2</xsl:element>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:variable name="nodeSizeScaleFactor" select="xalan:nodeset($treeSizeScaleFactor)" />

						<xsl:value-of select="round($nodeMinYNode/GUICoord[2]/@Y div ($nodeSizeScaleFactor/pd.size.scale.y * 2))" />
						
					</xsl:when>
					<xsl:when test="not($nodeMinYLink = '')">
						<xsl:variable name="treeSizeScaleFactor">					
							<xsl:element name="pd.size.scale.x">5</xsl:element>
							<xsl:element name="pd.size.scale.y">2</xsl:element>
						</xsl:variable>
						<xsl:variable name="nodeSizeScaleFactor" select="xalan:nodeset($treeSizeScaleFactor)" />
						<xsl:value-of select="round($nodeMinYLink/GUICoord[2]/@Y div ($nodeSizeScaleFactor/pd.size.scale.y * 2))" />
					</xsl:when>
					<xsl:when test="not($nodeMinYText = '')">
						<xsl:variable name="treeSizeScaleFactor">					
							<xsl:element name="pd.size.scale.x">5</xsl:element>
							<xsl:element name="pd.size.scale.y">2</xsl:element>							
						</xsl:variable>
						<xsl:variable name="nodeSizeScaleFactor" select="xalan:nodeset($treeSizeScaleFactor)" />
						<xsl:value-of select="round($nodeMinYText/GUICoord[2]/@Y div ($nodeSizeScaleFactor/pd.size.scale.y * 2))" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="'10'" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>

<!--
			<xsl:copy-of select="$NodeDiagramRegion " />
			<xsl:copy-of select="$nDiagramMinXWidth" />
			<xsl:copy-of select="$nDiagramMinYWidth" />
-->			

			<xsl:call-template name="writeDiagramGUISettings">
				<xsl:with-param name="NodeDiagramRegion" select="$NodeDiagramRegion" />
			</xsl:call-template>
			<xsl:value-of select="$newline" />			

			<xsl:element name="NodesGUI">
				<xsl:for-each select="Nodes/*">
					<xsl:call-template name="writeNodeGUI">
						<xsl:with-param name="NodeDiagramRegion" select="$NodeDiagramRegion" />
						<xsl:with-param name="nDiagramMinXWidth" select="$nDiagramMinXWidth" />
						<xsl:with-param name="nDiagramMinYWidth" select="$nDiagramMinYWidth" />
					</xsl:call-template>
				</xsl:for-each>				
			</xsl:element>

			<xsl:element name="LinksGUI">
				<xsl:for-each select="Links/DiagramTransitionElement">
					<xsl:call-template name="writeLinkGUI" >
						<xsl:with-param name="NodeDiagramRegion" select="$NodeDiagramRegion" />
						<xsl:with-param name="nDiagramMinXWidth" select="$nDiagramMinXWidth" />
						<xsl:with-param name="nDiagramMinYWidth" select="$nDiagramMinYWidth" />
					</xsl:call-template>
				</xsl:for-each>				
			</xsl:element>

			<xsl:element name="Texts">
				<xsl:for-each select="DiagramTextElement">
					<xsl:call-template name="writeText" >
						<xsl:with-param name="NodeDiagramRegion" select="$NodeDiagramRegion" />
						<xsl:with-param name="nDiagramMinXWidth" select="$nDiagramMinXWidth" />
						<xsl:with-param name="nDiagramMinYWidth" select="$nDiagramMinYWidth" />
					</xsl:call-template>
				</xsl:for-each>				
			</xsl:element>

		</xsl:element>
	</xsl:template>

	<!-- ***  write the GUISettings of the GUIDiagram*** -->
	<xsl:template name="writeDiagramGUISettings" >
		<xsl:param name="NodeDiagramRegion" />
		<GUISettings>
			<DiagramPermission>	
				<xsl:attribute name="Value">No</xsl:attribute>
			</DiagramPermission>
			<PageSettings>	
				<xsl:attribute name="SizeX">
					<xsl:value-of select="$NodeDiagramRegion/DiagramRegion/pd.max.x + 50" />
				</xsl:attribute>
				<xsl:attribute name="SizeY">
					<xsl:value-of select="$NodeDiagramRegion/DiagramRegion/pd.max.y + 50" />
				</xsl:attribute>
				<xsl:attribute name="BoundVisible">Yes</xsl:attribute>
			</PageSettings>
			<ComponentView>	
				<xsl:attribute name="Type">No</xsl:attribute>
				<xsl:attribute name="UIDs">No</xsl:attribute>
				<xsl:attribute name="Labels">Yes</xsl:attribute>
			</ComponentView>
			<Grid>	
				<xsl:attribute name="Snap">No</xsl:attribute>
				<xsl:attribute name="SizeX">0.240000</xsl:attribute>
				<xsl:attribute name="SizeY">0.240000</xsl:attribute>
				<xsl:attribute name="Visible">No</xsl:attribute>
				<Color>	
					<xsl:attribute name="B">0</xsl:attribute>
					<xsl:attribute name="G">0</xsl:attribute>
					<xsl:attribute name="R">0</xsl:attribute>					
				</Color>
			</Grid>
		</GUISettings>
	</xsl:template>

	<!-- *** write the NodeGUI element of NodesGUI *** -->
	<xsl:template name="writeNodeGUI" >
		<xsl:param name="NodeDiagramRegion" />
		<xsl:param name="nDiagramMinXWidth" select="1" />
		<xsl:param name="nDiagramMinYWidth" select="1" />
		<xsl:element name="NodeGUI">
			<xsl:call-template name="writeNodeLinkGUI">
				<xsl:with-param name="UniqID" select="@pd.id" />				
			</xsl:call-template>
						
			<xsl:element name="GUICoord2" >
			<!--
					<MinX>
					<xsl:value-of select="$NodeDiagramRegion/DiagramRegion/pd.min.x" />
					</MinX>
					<MinY>
					<xsl:value-of select="$NodeDiagramRegion/DiagramRegion/pd.min.y" />
					</MinY>
					<centerX>
					<xsl:value-of select="GUICoord[2]/@X" />
					</centerX>
					<centerY>
					<xsl:value-of select="GUICoord[2]/@Y" />
					</centerY>
					<X>
					<xsl:value-of select="GUICoord[1]/@X" />
					</X>
					<Y>
					<xsl:value-of select="GUICoord[1]/@Y" />
					</Y>
					<MinXID>
						<xsl:value-of select="$NodeDiagramRegion/DiagramRegion/pd.min.x/@pd.node.id" />
					</MinXID>
					<MinYID>
						<xsl:value-of select="$NodeDiagramRegion/DiagramRegion/pd.min.y/@pd.node.id" />
					</MinYID>
					<MinXwidth>
						<xsl:value-of select="$nDiagramMinXWidth" />
					</MinXwidth>
					<MinYwidth>
						<xsl:value-of select="$nDiagramMinYWidth" />
					</MinYwidth>

					-->

				<xsl:variable name="treeSizeScaleFactor">					
					<xsl:choose>
						<xsl:when test="@pd.node.name = 'InitialNode' or @pd.node.name = 'SuccessEndNode' or @pd.node.name = 'FailureEndNode' or @pd.node.name = 'BreakNode'">
							<xsl:element name="pd.size.scale.x">3</xsl:element>
							<xsl:element name="pd.size.scale.y">2.75</xsl:element>
						</xsl:when>
						<xsl:when test="@pd.node.name = 'OutboundServiceNode' or @pd.node.name = 'InboundServiceNode'">
							<xsl:element name="pd.size.scale.x">10</xsl:element>
							<xsl:element name="pd.size.scale.y">3</xsl:element>
						</xsl:when>
						<xsl:when test="@pd.node.name = 'IteratorNode' or @pd.node.name = 'SubDiagramNode'">
							<xsl:element name="pd.size.scale.x">5</xsl:element>
							<xsl:element name="pd.size.scale.y">2</xsl:element>
						</xsl:when>
						<xsl:otherwise>
							<xsl:element name="pd.size.scale.x">5</xsl:element>
							<xsl:element name="pd.size.scale.y">2</xsl:element>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:variable name="nodeSizeScaleFactor" select="xalan:nodeset($treeSizeScaleFactor)" />

				

				<xsl:variable name="nMinX" select="round((GUICoord[1]/@X - (3 * ((GUICoord[2]/@X div ($nodeSizeScaleFactor/pd.size.scale.x * 2)) - $NodeDiagramRegion/DiagramRegion/pd.min.x + (3 * $nDiagramMinXWidth)))) div 4)" />
				<xsl:variable name="nMinY" select="round((GUICoord[1]/@Y - (3 * ((GUICoord[2]/@Y div ($nodeSizeScaleFactor/pd.size.scale.y * 2)) - $NodeDiagramRegion/DiagramRegion/pd.min.y + (3 * $nDiagramMinYWidth)))) div 4)" />
				<xsl:element name="GUICoord">
					<xsl:attribute name="X">
						<xsl:value-of select="$nMinX" />
					</xsl:attribute>
					<xsl:attribute name="Y">
						<xsl:value-of select="$nMinY" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="GUICoord">
					<xsl:attribute name="X">
						<xsl:value-of select="$nMinX + round(GUICoord[2]/@X div ($nodeSizeScaleFactor/pd.size.scale.x * 2))" />
					</xsl:attribute>
					<xsl:attribute name="Y">
						<xsl:value-of select="$nMinY + round(GUICoord[2]/@Y div ($nodeSizeScaleFactor/pd.size.scale.y * 2))" />
					</xsl:attribute>
				</xsl:element>
								
			</xsl:element>

			<xsl:if test="@pd.node.name = 'DecisionNode'">
				<xsl:element name="DecisionNode">
					<xsl:attribute name="UniqID">
						<xsl:value-of select="@pd.id" />
					</xsl:attribute>

					<Label>
						<xsl:value-of select="@Label" />
					</Label>
					<Desc>							
						<xsl:value-of select="Desc" />
					</Desc>

					<xsl:variable name="nodeLinks" select="../../Links" />

					<DecisionBranches>
						<xsl:for-each select="pd.decision.branches/pd.decision.branch">
							<xsl:variable name="thisBranch.xmi.id" select="@xmi.id" />
							
							<xsl:if test="position() &lt;= 7">
								<xsl:element name="DecisionBranch">
									<xsl:attribute name="Type">
										<xsl:value-of select="type" />
									</xsl:attribute>
									<Condition>
										<xsl:value-of select="condition" />
									</Condition>
									<Comment>
										<xsl:value-of select="comment" />
									</Comment>
									<Label>
										<xsl:value-of select="label" />
									</Label>
									<LinkID>
										<xsl:value-of select="$nodeLinks/DiagramTransitionElement[@xmi.id = $thisBranch.xmi.id]/@pd.id" />										
									</LinkID>
								</xsl:element>
							</xsl:if>
						</xsl:for-each>
						<xsl:if test="count(pd.decision.branches/pd.decision.branch) &lt; 7">
							<xsl:call-template name="writeUnusedBranches">
								<xsl:with-param name="nThisBranch" select="count(pd.decision.branches/pd.decision.branch) + 1" />
							</xsl:call-template>							
						</xsl:if>
					</DecisionBranches>
				</xsl:element>									
			</xsl:if>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeUnusedBranches">
	<xsl:param name="nThisBranch" />
		<xsl:choose>
			<xsl:when test="$nThisBranch &lt; = 7">
				<DecisionBranch Type="NotUsed"> <Condition />  <Comment />  <Label />  <LinkID /> </DecisionBranch>				

				<xsl:call-template name="writeUnusedBranches">
					<xsl:with-param name="nThisBranch" select="$nThisBranch+ 1" />
				</xsl:call-template>							

			</xsl:when>
			<xsl:otherwise>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>							
						
	<!-- *** write the LinkGUI element of LinksGUI *** -->
	<xsl:template name="writeLinkGUI" >
		<xsl:param name="NodeDiagramRegion" />
		<xsl:param name="nDiagramMinXWidth" select="1" />
		<xsl:param name="nDiagramMinYWidth" select="1" />
		<xsl:element name="LinkGUI">
			<xsl:attribute name="IsOrthogonal">
				<xsl:value-of select="'No'" />
			</xsl:attribute>

			<xsl:call-template name="writeNodeLinkGUI">
				<xsl:with-param name="UniqID" select="@pd.id" />				
			</xsl:call-template>
						
			<xsl:element name="GUICoords" >
				<xsl:variable name="nMinX" select="round((GUICoord[1]/@X - (3 * ((GUICoord[2]/@X div (3 * 2)) - $NodeDiagramRegion/DiagramRegion/pd.min.x + (3 * $nDiagramMinXWidth)))) div 4)" />
				<xsl:variable name="nMinY" select="round((GUICoord[1]/@Y - (3 * ((GUICoord[2]/@Y div (3 * 2)) - $NodeDiagramRegion/DiagramRegion/pd.min.y + (3 * $nDiagramMinYWidth)))) div 4)" />
				<xsl:element name="GUICoord">
					<xsl:attribute name="X">
						<xsl:value-of select="$nMinX" />
					</xsl:attribute>
					<xsl:attribute name="Y">
						<xsl:value-of select="$nMinY" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="GUICoord">
					<xsl:attribute name="X">
						<xsl:value-of select="$nMinX + round(GUICoord[2]/@X div (3 * 2))" />
					</xsl:attribute>
					<xsl:attribute name="Y">
						<xsl:value-of select="$nMinY + round(GUICoord[2]/@Y div (3 * 2))" />
					</xsl:attribute>
				</xsl:element>				
			</xsl:element>

			<xsl:if test="not(@pd.id.GUITo = @pd.id.ServerTo) or not(@pd.id.GUIFrom = @pd.id.ServerFrom)">
				<xsl:element name="Connect">
					<xsl:attribute name="To">
						<xsl:value-of select="@pd.id.GUITo" />
					</xsl:attribute>
					<xsl:attribute name="From">
						<xsl:value-of select="@pd.id.GUIFrom" />
					</xsl:attribute>				
				</xsl:element> 
				<xsl:if test="@pd.id.ServerTo = 'DONOTWRITE'">
					<xsl:element name="DecisionLink">
						<xsl:attribute name="UniqID">
							<xsl:value-of select="@pd.id" />
						</xsl:attribute>

						<Label>
							<xsl:value-of select="@Label" />
						</Label>
						<Desc>							
							<xsl:value-of select="Desc" />
						</Desc>
					</xsl:element>					
				</xsl:if>
			</xsl:if>

		</xsl:element>
	</xsl:template>

	<!-- *** write the Text element of Texts *** -->
	<xsl:template name="writeText" >
		<xsl:param name="NodeDiagramRegion" />
		<xsl:param name="nDiagramMinXWidth" select="1" />
		<xsl:param name="nDiagramMinYWidth" select="1" />
		<xsl:element name="Text">
			<TextValue>
				<xsl:value-of select="TextValue" />
			</TextValue>

			<xsl:copy-of select="GUIFont" />
			<xsl:value-of select="$newline" />
						
			<xsl:element name="GUICoord2" >
				<xsl:variable name="nMinX" select="round((GUICoord[1]/@X - (3 * ((GUICoord[2]/@X div (3 * 2)) - $NodeDiagramRegion/DiagramRegion/pd.min.x + (3 * $nDiagramMinXWidth)))) div 4)" />
				<xsl:variable name="nMinY" select="round((GUICoord[1]/@Y - (3 * ((GUICoord[2]/@Y div (3 * 2)) - $NodeDiagramRegion/DiagramRegion/pd.min.y + (3 * $nDiagramMinYWidth)))) div 4)" />
				<xsl:element name="GUICoord">
					<xsl:attribute name="X">
						<xsl:value-of select="$nMinX" />
					</xsl:attribute>
					<xsl:attribute name="Y">
						<xsl:value-of select="$nMinY" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="GUICoord">
					<xsl:attribute name="X">
						<xsl:value-of select="$nMinX + round(GUICoord[2]/@X div (3 * 2))" />
					</xsl:attribute>
					<xsl:attribute name="Y">
						<xsl:value-of select="$nMinY + round(GUICoord[2]/@Y div (3 * 2))" />
					</xsl:attribute>
				</xsl:element>				
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<!-- *** write the standard NodeLinkGUI element*** -->
	<xsl:template name="writeNodeLinkGUI" >
		<xsl:param name="UniqID" />
		<xsl:element name="NodeLinkGUI" >
			<xsl:attribute name="UniqIDRef">
				<xsl:value-of select="$UniqID" />
			</xsl:attribute>

			<xsl:copy-of select="GUIFont" />
			<xsl:value-of select="$newline" />
		</xsl:element>
	</xsl:template>

	<!-- *** write the standard ModelActivityElement element*** -->
	<xsl:template name="writeModelActivityElement" >
		<xsl:param name="UniqueID" />
		<xsl:element name="ModelActivityElement">
			<xsl:attribute name="UniqID">
				<xsl:value-of select="$UniqueID" />
			</xsl:attribute>
			<xsl:element name="ModelElement"> 
				<xsl:attribute name="Version">
					<xsl:value-of select="'1'" />
				</xsl:attribute>

				<Desc>
					<xsl:value-of select="Desc" />
				</Desc>
			</xsl:element>	
			
			<xsl:element name="Label">			
				<xsl:value-of select="@Label" />
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<!-- *** create the standard GUIFont element*** -->
	<xsl:template name="CreateGUIFont" >
	<xsl:param name="strStyle" select="normalize-space('FillColor.Blue= 255,FillColor.Green= 255,FillColor.Red= 255,FillColor.Transparent=0,Font.Blue= 0,Font.Green= 0,Font.Red= 0,Font.FaceName=Arial,Font.Size= 10,Font.Bold=0,Font.Italic=0,Font.Strikethrough=0,Font.Underline=0,LineColor.Blue= 51,LineColor.Green= 0,LineColor.Red= 153,')" />		
		<xsl:element name="GUIFont">
			<xsl:attribute name="Italic">
				<xsl:call-template name="extractFontProperties">				
					<xsl:with-param name="strStyle" select="$strStyle" />
					<xsl:with-param name="strPropertyName" select="'Font.Italic='" />
					<xsl:with-param name="bConvertToYesNO" select="'true'" />
				</xsl:call-template>
			</xsl:attribute>
			<xsl:attribute name="StrikeOut">
				<xsl:call-template name="extractFontProperties">				
					<xsl:with-param name="strStyle" select="$strStyle" />
					<xsl:with-param name="strPropertyName" select="'Font.Strikethrough='" />
					<xsl:with-param name="bConvertToYesNO" select="'true'" />
				</xsl:call-template>
			</xsl:attribute>
			<xsl:attribute name="UnderLine">
				<xsl:call-template name="extractFontProperties">				
					<xsl:with-param name="strStyle" select="$strStyle" />
					<xsl:with-param name="strPropertyName" select="'Font.Underline='" />
					<xsl:with-param name="bConvertToYesNO" select="'true'" />
				</xsl:call-template>
			</xsl:attribute>

			<FaceName>
				<xsl:call-template name="extractFontProperties">				
					<xsl:with-param name="strStyle" select="$strStyle" />
					<xsl:with-param name="strPropertyName" select="'Font.FaceName='" />
				</xsl:call-template>
			</FaceName>

			<PointSize>
				<xsl:call-template name="extractFontProperties">				
					<xsl:with-param name="strStyle" select="$strStyle" />
					<xsl:with-param name="strPropertyName" select="'Font.Size='" />
				</xsl:call-template>
			</PointSize>

			<Height>
				<xsl:value-of select="'10'" />
			</Height>	
			
			<Color>	
				<xsl:attribute name="B">
					<xsl:call-template name="extractFontProperties">				
						<xsl:with-param name="strStyle" select="$strStyle" />
						<xsl:with-param name="strPropertyName" select="'Font.Blue='" />
					</xsl:call-template>
				</xsl:attribute>
				<xsl:attribute name="G">
					<xsl:call-template name="extractFontProperties">				
						<xsl:with-param name="strStyle" select="$strStyle" />
						<xsl:with-param name="strPropertyName" select="'Font.Green='" />
					</xsl:call-template>
				</xsl:attribute>
				<xsl:attribute name="R">
					<xsl:call-template name="extractFontProperties">				
						<xsl:with-param name="strStyle" select="$strStyle" />
						<xsl:with-param name="strPropertyName" select="'Font.Red='" />
					</xsl:call-template>
				</xsl:attribute>
			</Color>							
		</xsl:element>
	</xsl:template>

	<!-- *** extract font properties from a string *** -->
	<xsl:template name="extractFontProperties">				
		<xsl:param name="strStyle" />
		<xsl:param name="strPropertyName" />
		<xsl:param name="bConvertToYesNO" select="'false'" />
		<xsl:variable name='strValue' select="normalize-space(substring-before(substring-after($strStyle, $strPropertyName), ','))" />
		<xsl:choose>
			<xsl:when test="$bConvertToYesNO = 'true'">
				<xsl:choose>
					<xsl:when test="$strValue = '0' or $strValue = ''">
						<xsl:value-of select="'No'" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="'Yes'" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select='$strValue' />				
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- *** extract GUI Coordination from a geometry string *** -->
	<xsl:template name="CreateGUICoord">				
		<xsl:variable name="strGeometry" select="normalize-space(@geometry)" />
		<xsl:variable name="nMinX" select="normalize-space(substring-before($strGeometry, ','))" />
		<xsl:variable name="nMinY" select="normalize-space(substring-before(substring-after($strGeometry, ','), ','))" />
		<xsl:variable name="nWidth" select="normalize-space(substring-before(substring-after(substring-after($strGeometry, ','), ','), ','))" />
		<xsl:variable name="nHeight" select="normalize-space(substring-before(substring-after(substring-after(substring-after($strGeometry, ','), ','), ','), ','))" />
		<xsl:element name="GUICoord">
			<xsl:attribute name="X">
				<xsl:value-of select="$nMinX" />
			</xsl:attribute>
			<xsl:attribute name="Y">
				<xsl:value-of select="$nMinY" />
			</xsl:attribute>
		</xsl:element>
		<xsl:element name="GUICoord">
			<xsl:attribute name="X">
				<xsl:value-of select="$nWidth " />
			</xsl:attribute>
			<xsl:attribute name="Y">
				<xsl:value-of select="$nHeight" />
			</xsl:attribute>
		</xsl:element>
	</xsl:template>
	
</xsl:stylesheet>
